/***************************************************************************
 *   Copyright (C) 2008 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
// Types and prototypes for common functions
#ifndef _COMMON_H
#define _COMMON_H
#include <stdbool.h>
#include <stdio.h>
#include "types.h"
#include "c4.5/defns.i"
#include "c4.5/types.i"
#include "c4.5/extern.i"

//#define AUTOTEST      // The code checks for errors

#define REMOVE_NEWLINE(str)             {if(str[strlen(str)-1] == '\n') str[strlen(str)-1] = '\0';}

// Error codes
#define ERR_NONE                0
#define ERR_NOT_YET_IMPLEMENTED 1
#define ERR_UNSPECIFIED         2
#define ERR_FILE                3
#define ERR_MEMORY              4
#define ERR_READ                5
#define ERR_PARAM               6       // Some of the parameters conflict or are inacurate
#define ERR_PVM                 7
#define ERR_NUMERICAL           8
#define ERR_UNSPECTED_RESULT	9

#define MAX_FILE_NAME   10000

// Prototypes
int LoadInteger(FILE *, char *, int, FILE *);
scalar LoadReal(FILE *, char *, scalar, FILE *);
bool LoadBoolean(FILE *, char *, bool, FILE *);
int LoadEnumerate(FILE *, char *, char *[], int, int, FILE *);
void LoadString(FILE *, char *, char *, char *, FILE *);
double ReadData(scalar ** *, int *, int *, int *, char *);
void ReadDataAndLabels(scalar ***, int **, int *, int *, int *, char *, bool);
void ReadLimitedDataAndLabels(scalar ***, int **, int *, int *, int *, char *, int);
void ReadDataInfo(int *, int *, int *, char *);
int ReadNextInstanceAndLabel(scalar *, int *, int *, int *, FILE *, bool *);
int ReadNextBlockOfInstancesAndLabels(scalar **, int *, int *, int *, FILE *, bool *, int);
void SortClasses(scalar **, int, int *, int, int);
void TypifyInputData(scalar **, scalar **, int, int, int);
void ScaleInputData(scalar **, scalar **, int, int, int, scalar, scalar, char *);
void OnErrorExit(int, char *);
void GetRandomisedVector(int *, int);
void ZeroVector(scalar *, int *, int);
void ZeroMatrix(scalar **, int **, int, int);
void RandomizeVector(int *, int);
void GenerateRandomPopulation(int **, int, int, scalar);
scalar ReadDataAndLabelsInSparseFormat(scalar ***, int **, int *, int *, int *, char *, int ***, bool);
void ReadLimitedSparseDataAndLabels(scalar ***, int **, int *, int *, int *, char *, int);
scalar ReadSparseDataAndLabelsInSparseFormat(scalar ***, int **, int *, int *, int *, char *, int ***, bool);
bool GetNextPair(char *, int *, int *, scalar *, int *);
void ReadSubsetWithDirectAccess(scalar **, int *, int, int, char *, long int **, int, int *);

void PrintTwoClassMeasures(scalar, scalar, scalar, scalar, FILE *, char *);

int CompareFeat(const void *, const void *);
int CompareWInv(const void *, const void *);
int CompareW(const void *, const void *);
int CompareRank(const void *, const void *);
int CompareIndex(const void *, const void *);
int CompareDouble(const void *, const void *);
int CompareInt(const void *, const void *);

void PrintMulticlassMeasures(int **, int, int, FILE *, char *);

void Trim(char *);

char *ToLower(char *);

bool FindFile(char *, char *, char *);
void FindFileRecursive(char *, char *, char *, bool *);

void BubbleSort(int *, int);
scalar EuclideanDistance(scalar *, scalar *, int);

// Principal components analysis
void PrincipalComponentsAnalysis(scalar **, int, int, int, scalar ** *, scalar **);
void TypifyData(scalar **, scalar **, scalar **, int, int);
void Variance(scalar **, scalar **, int, int);
void MeanCorrect(scalar **, scalar **, int, int);
void Centroid(scalar **, int, int, scalar *);
void Correlation(scalar **, scalar **, scalar **, int, int);
void CrossProducts(scalar **, scalar **, int s, int v);
void MatrixProduct(scalar **, scalar **, scalar **, int, int, int);
void EigenValuesVectors(scalar *, scalar *, int, scalar **, int *);
void TridiagonalizeHH(scalar **, scalar *, scalar *, int);
scalar Sign2(scalar, scalar);
void ReadFastaData(scalar ** *, int **, int *, int *, char *, char *, bool, int, int, bool, bool);
int ReadNextBlockOfFastaRawSequences(scalar **, int *, char *, char *, int, int, int, bool);
void ReadSparseDataAndLabels(scalar ** *, int **, int *, int *, int *, char *, bool);
scalar ReadBinaryDataAndLabels(scalar ***, int **, int *, int *, int *, char *, int ***, bool, bool);

scalar EuclideanDistanceSubspace(scalar *, scalar *, int, int *);

void AllocateScalarMatrix(scalar ***, int, int);
void FreeScalarMatrix(scalar ***, int);
void AllocateIntegerMatrix(int ***, int, int);
void FreeIntegerMatrix(int ***, int);

// C4.5 prototypes
Boolean ReadName(FILE *, String);
void GetNames(void);
void GetData(void);
void Error(short, String, String);
Description GetDescription(int);
void OneTree(bool, FILE *, scalar *);
ClassNo Category(Description, Tree);
void SoftenThresh(Tree, FILE *, scalar *);
void Evaluate(Boolean, short, FILE *);
Tree CopyTree(Tree);
void InitialiseTreeData(void);
void InitialiseWeights(scalar *);
void FormTarget(ItemNo);
void FormInitialWindow(void);
void Shuffle(void);
void PrintConfusionMatrix(ItemNo *);
void ScanTree(Tree, ItemNo, ItemNo, FILE *);
ItemNo Group(DiscrValue, ItemNo, ItemNo, Tree);
void Quicksort(ItemNo, ItemNo, Attribute, void (*)());
void EvalSubset(Attribute, ItemNo, ItemNo, ItemCount);
void EvalDiscreteAtt(Attribute, ItemNo, ItemNo, ItemCount);
void EvalContinuousAtt(Attribute, ItemNo, ItemNo);
void SubsetTest(Tree, Attribute);
void DiscreteTest(Tree, Attribute);
void ContinTest(Tree, Attribute);
void Show(Tree, short, FILE *);
void ShowBranch(short, Tree, DiscrValue, FILE *);
void Indent(short, char *, FILE *);
int TreeSize(Tree);
void ReleaseTree(Tree);
void OutTree(Tree);
void SaveDiscreteNames();
void StreamOut(String, int);
void StreamIn(String, int);
void RecoverDiscreteNames();
void Classify(Description, Tree, float);
void Sprout(Tree, DiscrValue);
void ComputeFrequencies(Attribute, ItemNo, ItemNo);
void PrintDistribution(Attribute, DiscrValue, Boolean);
void Combine(DiscrValue, DiscrValue, DiscrValue);
void Uncombine(DiscrValue, DiscrValue);
void PrintSubset(Attribute, Set);
void ResetFreq(DiscrValue);
void CheckPossibleValues(Tree);
void PrintTree(Tree, FILE *);
float AddErrs(ItemCount, ItemCount);
short BestTree(FILE *, scalar *);
void ReleaseTreeData(void);
Boolean Prune(Tree, FILE *, scalar *);
void SaveTree(Tree, String, String, FILE *);
Tree GetTree(String, String);
float EstimateErrors(Tree, ItemNo, ItemNo, short, Boolean, FILE *);
Tree Iterate(ItemNo, ItemNo, FILE *, scalar *);
int Which(String, String [], short, short);
String CopyString(String);
void Error(short, String, String);
float TotalInfo(ItemCount [], DiscrValue, DiscrValue);
float DiscrKnownBaseInfo(DiscrValue, ItemCount);
void EvalDiscreteAtt(Attribute, ItemNo, ItemNo, ItemCount);
void ComputeFrequencies(Attribute, ItemNo, ItemNo);
void DiscreteTest(Tree, Attribute);
void EvalContinuousAtt(Attribute, ItemNo, ItemNo);
void ContinTest(Tree, Attribute);
float GreatestValueBelow(Attribute, float);
Tree FormTree(ItemNo, ItemNo);
ItemCount CountItems(ItemNo, ItemNo);
void Swap(ItemNo, ItemNo);
void FormInitialWindow(void);
void Shuffle(void);
float Worth(float, float, float);
float ComputeGain(float, float, DiscrValue, ItemCount);
short MaxLine(Tree, FILE *);
Tree InTree(void);
Tree Leaf(ItemCount *, ClassNo, ItemCount, ItemCount);

// PVM
int PVMSpawn(int, char *, int *, char *[], int *, char *, bool);
int InitGuardian(int, int *, char *, char *);

#endif  // _COMMON_H
