/***************************************************************************
 *   Copyright (C) 2008 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <sys/types.h>
#include <sys/dir.h>
#include <stdio.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include "common.h"


#define MAX_NAME_LENGTH 1024

void FindFileRecursive(char *pattern, char *folder, char *path, bool *found)
{
    DIR *dir;   /* pointer to the scanned directory. */
    struct dirent *entry; /* pointer to one directory entry.   */
    struct stat dir_stat;
    char file_name[MAX_NAME_LENGTH];

    if ((dir = opendir(folder)) == NULL)
        OnErrorExit(ERR_FILE, "Cannot open datasets directory");

    // Scan the directory, traversing each sub-directory, and matching the pattern for each file name
    while ((entry = readdir(dir)) && !(*found)) {
        // Check if the pattern matchs
        if (entry->d_name && !strcmp(entry->d_name, pattern)) {
            sprintf(path, "%s/%s", folder, entry->d_name);
            *found = true;
            closedir(dir);
            return;
        }

        // Ignore
        if (strcmp(entry->d_name, ".") == 0)
            continue;
        if (strcmp(entry->d_name, "..") == 0)
            continue;

        sprintf(file_name, "%s/%s", folder, entry->d_name);
        stat(file_name, &dir_stat);
        if (S_ISDIR(dir_stat.st_mode))
            FindFileRecursive(pattern, file_name, path, found);
    }

    closedir(dir);
}

bool FindFile(char *pattern, char *folder, char *path)
{
    bool found = false;

    FindFileRecursive(pattern, folder, path, &found);

    return found;
}
