// Generic macros.
#ifndef _MACROS_H
#define _MACROS_H

#include <malloc.h>
#include <stdlib.h>
#include <stdbool.h>
#include "common.h"

/* MACROS. */
/* Generic definitions. */
#define YES             1
#define NO              0
#define ZERO            0.000001
#define ZERO2           0.000000000001
#define INFINITE        10e20

// Minimum size of memory allocation in bytes
#define MIN_ALLOCATION  10

// Relief algorithm: maximum ((unsigned int) number) of features that allows to use a cache (so there is enough memory)
#define MAX_FEAT_MEM 25

/* Macros*/
#define SHOWPROGRESS(show, format, ...) {               \
    if (show)                         \
        fprintf(stdout, format, ##__VA_ARGS__); fflush(stdout);  \
}

#define EQUAL(x, y)      (fabs((scalar) x - (scalar) y) < ZERO)

#define RANDOM(a, b)     (((((double) random()-1.0)/(RAND_MAX))*((b)-(a)))+(a))       /* a <= x  < b */

#define MIN(a, b)        (((a)>(b)) ? (b) : (a))
#define MAX(a, b)        (((a)<(b)) ? (b) : (a))

#define MALLOC(pt, type, number, str)   {{if ((((unsigned int) number) != 0)) { \
                                        if (((pt) = (type *) \
                                           malloc (((size_t) (MAX(sizeof (type) *((((unsigned int) number))), MIN_ALLOCATION))))) \
                                           == NULL) \
                                          OnErrorExit (ERR_MEMORY, (str)); } \
                                        else \
                                          pt = NULL;}}

#define CALLOC(pt, type, number, str)   {if (((unsigned int) number)) { \
                                        if (((pt) = (type *) \
                                           calloc (sizeof (type),((unsigned int) number))) \
                                           == NULL) \
                                          OnErrorExit (ERR_MEMORY, (str)); } \
                                        else \
                                          pt = NULL;}

#define REALLOC(pt, type, number, str)  {if ((((unsigned int) number))) { \
                                        if (((pt) = (type *) \
                                           realloc (pt, sizeof (type) *(((unsigned int) number)))) \
                                           == NULL) \
                                          OnErrorExit (ERR_MEMORY, (str)); }\
                                        else \
                                          pt = NULL;}

#define ALLOCA(pt, type, number, str)   {if (((unsigned int) number)) { \
                                        if (((pt) = (type *) \
                                           alloca (MAX(MIN_ALLOCATION, sizeof (type) *(((unsigned int) number))))) \
                                           == NULL) \
                                          OnErrorExit (ERR_MEMORY, (str)); }\
                                        else \
                                          pt = NULL;}

#define BOUND(m, value, M)              {if (value < (m)) value = (m); else if (value > (M)) value = (M);}

#define ASSERT(pred, err, str)          {if (!(pred)) OnErrorExit (err, str);}

#define ABS(x)                          ((x)>=0 ? (x) : -(x))

#define FREE(x)                         {if ((x)==NULL) OnErrorExit(ERR_MEMORY, "Attempt to free null pointer"); free (x); (x) = NULL;}

static inline int WIDTH(int x) {
    bool power_of_ten;
    double exponent;

    power_of_ten = !modf(log10(fabs(x)), &exponent);

    int width = (int) ceil(log10((x))) + (power_of_ten ? 1 : 0);

    // Limit [1, 10]
    BOUND(1, width, 10);

    return width;
}

#endif                          /* !_MACROS_H */
