/***************************************************************************
 *   Copyright (C) 2008 by Nicolas Garcia-Pedrajas   *
 *   npedrajas@uco.es   *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include "types.h"
#include "macros.h"
#include "common.h"

void OnErrorExit(int id, char *message)
{
    char *codes[] = {"No error", "Not yet implemented", "Unspecified error", "File error", "Memory error", "Read failed", "Invalid arguments", "PVM error", "Numerical error", "Unexpected result"};

    printf("%s: %s\n", codes[id], message);

    exit(id);
}

// void OnErrorExit(int id, char *message)
// {
//     FILE *f;
//     char *codes[] = {"No error", "Not yet implemented", "Unspecified error", "File error", "Memory error", "Read failed", "Invalid arguments", "PVM error", "Numerical error", "Unexpected result"};
//
//     f = fopen (tempnam(".", NULL), "w");
//     fprintf(f, "%s: %s\n", codes[id], message);
//     fclose(f);
//
//     exit(id);
// }



void RandomizeVector(int *v, int n)
{
    int l, r;

    for (int i = 0; i < n; i++) {
        // Exchange two random positions
        l = RANDOM(0, n);
        r = RANDOM(0, n);

        int aux = v[l];
        v[l] = v[r];
        v[r] = aux;
    }
}

void ZeroVector(scalar *v, int *w, int n)
{
    if (v)
        for (int i = 0; i < n; i++)
            v[i] = 0.0;
    else
        for (int i = 0; i < n; i++)
            w[i] = 0;
}

void ZeroMatrix(scalar **v, int **w, int f, int c)
{
    if (v)
        for (int i = 0; i < f; i++)
            for (int j = 0; j < c; j++)
                v[i][j] = 0.0;
    else
        for (int i = 0; i < f; i++)
            for (int j = 0; j < c; j++)
                w[i][j] = 0;
}

// Returns a vector with its elements randomized
void GetRandomisedVector(int *v, int n)
{
    int aux, r;

    // Order
    for (int i = 0; i < n; i++)
        v[i] = i;

    // Randomize
    for (int i = 0; i < n; i++) {
        r = RANDOM(0, n);

        aux = v[r];
        v[r] = v[i];
        v[i] = aux;
    }

}

// Generates a random population of pop_size individuals and n_feat features selected with a feat_sel_prob probability
void GenerateRandomPopulation(int **new_pop, int pop_size, int n_feat,
                              scalar feat_sel_prob)
{
    int p1 = 0, null_ind;
    scalar local_pfeat = 0.0;

    for (int i = 0; i < pop_size; i++) {
        null_ind =  1;
        for (int j = 0; j < n_feat; j++) {
            //Get scalar random value between (0,1) in order to decide if we are going to select a feature
            local_pfeat = RANDOM(0, 1);

            // Select feature
            if (local_pfeat < feat_sel_prob) {
                new_pop[i][j] = 1;
                null_ind = 0;
            } else
                new_pop[i][j] = 0;
        }
        if (null_ind) {
            p1 = (int) RANDOM(0, n_feat);
            new_pop[i][p1] = 1;
        }

    }
}

// Auxiliary function for qsort()
int CompareDouble(const void *x, const void *y)
{
    scalar diff;

    if ((diff = * (scalar *) x - * (scalar *) y) > 0)
        return +1;
    else {
        if (diff < 0)
            return -1;
        else
            return 0;
    }
}

// Auxiliary function for qsort()
int CompareInt(const void *x, const void *y)
{
    if (* (int *) x < * (int *) y)
        return -1;
    else {
        if (* (int *) x > * (int *) y)
            return +1;
        else
            return 0;
    }
}

char *ToLower(char *p)
{
    for (int i = 0; i < strlen(p); i++)
        p[i] = tolower(p[i]);

    return p;
}

scalar EuclideanDistanceSubspace(scalar *v, scalar *w, int n, int *subspace)
{
    scalar dist = 0.0;

    for (int i = 0; i < n; i++)
        if (subspace[i])
            dist += (v[i] - w[i]) * (v[i] - w[i]);

    return sqrt(dist);
}

void AllocateScalarMatrix(scalar ***m, int r, int c)
{
    MALLOC(*m, scalar *, r, "AllocateScalarMatrix: *m");
    for (int i = 0; i < r; i++)
        MALLOC((*m)[i], scalar, c, "AllocateScalarMatrix: (*m)[i]");
}

void FreeScalarMatrix(scalar ***m, int r)
{
    for (int i = 0; i < r; i++)
        FREE((*m)[i]);
    FREE(*m);
}

void AllocateIntegerMatrix(int ***m, int r, int c)
{
    MALLOC(*m, int *, r, "AllocateIntegerMatrix: *m");
    for (int i = 0; i < r; i++)
        MALLOC((*m)[i], int, c, "AllocateIntegerMatrix: (*m)[i]");
}

void FreeIntegerMatrix(int ***m, int r)
{
    for (int i = 0; i < r; i++)
        FREE((*m)[i]);
    FREE(*m);
}
