/***************************************************************************
 *   Copyright (C) 2017 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FItnESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <math.h>
#include "common.h"
#include "types.h"

#define ZERO 0.000001

void PrintTwoClassMeasures(scalar tp, scalar tn, scalar fp, scalar fn, FILE *out_file, char *text)
{
    scalar specificity, sensitivity, precision, false_neg_rate,  ccr, mcc, recall, tp_rate, tn_rate, poc, f_measure, f1_score, ppv, npv, fpr, fall_out, fdr, acp;

    specificity = (tn + fp) > ZERO ? tn/(tn + fp) : 0.0;
    sensitivity = (tp + fn) > ZERO ? tp/(tp + fn) : 0.0;
    precision = (tp + fp) > ZERO ? tp/(tp + fp) : 0.0;
    false_neg_rate = (tp+fn > ZERO) ?  fn/(tp+fn) : 0.0;
    ccr = 0.5 * (specificity + sensitivity);
    mcc = ((tp)+(fp))*((tp)+(fn))*((tn)+(fp))*((tn)+(fn)) > 0.0 ? ((tn)*(tp)-(fn)*(fp))/sqrt(((tp)+(fp))*((tp)+(fn))*((tn)+(fp))*((tn)+(fn))) : 0.0;
    f_measure = f1_score = sensitivity+precision > ZERO ? (2.0*sensitivity*precision)/(sensitivity+precision) : 0.0;
    npv = tn+fn > ZERO ? tn/(tn+fn) : 0.0;
    fpr = fall_out = (fp + tn) > ZERO ? fp/(fp + tn) : 0.0;
    ppv = precision;
    fdr = 1.0 - ppv;
    tp_rate = recall = poc = sensitivity;
    tn_rate = specificity;

    // Print tokens
    fprintf(out_file, "@%s TP: %.0lf\n", text, tp);
    fprintf(out_file, "@%s FN: %.0lf\n", text, fn);
    fprintf(out_file, "@%s TN: %.0lf\n", text, tn);
    fprintf(out_file, "@%s FP: %.0lf\n", text, fp);
    // Specificity or tn rate
    fprintf(out_file, "@%s Specificity: %lf\n", text, specificity);
    fprintf(out_file, "@%s TN-rate: %lf\n", text, tn_rate);
    // Sensitivity, true positive rate, recall or probability of detection
    fprintf(out_file, "@%s Sensitivity: %lf\n", text, sensitivity);
    fprintf(out_file, "@%s Recall: %lf\n", text, recall);
    fprintf(out_file, "@%s TP-rate: %lf\n", text, tp_rate);
    fprintf(out_file, "@%s POC: %lf\n", text, poc);
    fprintf(out_file, "@%s G-mean: %lf\n", text, pow(sensitivity * specificity, 0.5));
    // F measure or F1 score
    fprintf(out_file, "@%s F-measure: %lf\n", text, f_measure);
    fprintf(out_file, "@%s F1-score: %lf\n", text, f1_score);
    // Precision or positive predictive value (PPV)
    fprintf(out_file, "@%s Precision: %lf\n", text, precision);
    fprintf(out_file, "@%s PPV: %lf\n", text, ppv);
    // Negative predictive value (NPV)
    fprintf(out_file, "@%s NPV: %lf\n", text, npv);
    // fp rate or fall out
    fprintf(out_file, "@%s FP-rate: %lf\n", text, fpr);
    fprintf(out_file, "@%s Fall-out: %lf\n", text, fall_out);
    fprintf(out_file, "@%s FN-rate: %lf\n", text, false_neg_rate);
    // False discovery rate
    fprintf(out_file, "@%s FDR: %lf\n", text, fdr);
    fprintf(out_file, "@%s BAC: %lf\n", text, (specificity+sensitivity)/2.0);
    fprintf(out_file, "@%s CCR: %lf\n", text, ccr);
    // Matthews correlation coefficient (MCC)
    fprintf(out_file, "@%s MCC: %lf\n", text, mcc);
    // Corrected classification rate
    fprintf(out_file, "@%s CC: %lf\n", text, sqrt((tp+fn)*(tn+fp)*(tp+fp)*(tn+fn)) > 0.0 ? ((tp*tn)-(fn*fp))/sqrt((tp+fn)*(tn+fp)*(tp+fp)*(tn+fn)) : 0.0);
    // Informedness
    fprintf(out_file, "@%s Informedness: %lf\n", text, tp_rate+specificity-1.0);
    // Markedness
    fprintf(out_file, "@%s Markedness: %lf\n", text, ppv+npv-1.0);
    fprintf(out_file, "@%s ACP: %lf\n", text, acp = ((tp/(tp+fn))+(tp/(tp+fp))+(tn/(tn+fp))+(tn/(tn+fn))) > 0.0 ? 0.25*((tp/(tp+fn))+(tp/(tp+fp))+(tn/(tn+fp))+(tn/(tn+fn))) : 0.0);
    fprintf(out_file, "@%s AC: %lf\n", text, (acp-0.5)*2.0);
    fflush(out_file);
}
