/***************************************************************************
 *   Copyright (C) 2008 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _TYPES_H
#define _TYPES_H

typedef unsigned char byte;

// Scalar can be {double, float}
#define SCALAR_IS_DOUBLE
// #define SCALAR_IS_FLOAT

// For PVM
#ifdef SCALAR_IS_FLOAT
typedef float scalar;
#define pvm_pkscalar    pvm_pkfloat
#define pvm_upkscalar   pvm_upkfloat
#define SCALAR_FORMAT   "%f"
#else
typedef double scalar;
#define pvm_pkscalar    pvm_pkdouble
#define pvm_upkscalar   pvm_upkdouble
#define SCALAR_FORMAT   "%lf"
#endif


// Types common to different programs

// Instance selection algorithms
typedef enum {
    NoIS, DROP3, ICF, CHC, MSS, RNN, IB3, RANDOMSELECTION, PSRCG, RC45, CFilter, PSA, CNN, CNNTL, OSS, SMOTE, ENTROPY, Undersampling, GCNN, BoostBased, RMHC, HMNEI, RNGE, LVQPRU, SimulatedAnnealing,
    kNNCHC
} t_ISAlgorithm;
#define RANDOMIS        7       // Algorithms to consider for random selection

// Feature selection algorithms
typedef enum {
    fsNONE, fsGA, fsLVF, fsFOCUS, fsRELIEF, fsRFE, fsCLASSIFIER, fsRANDOM, fsABB, fsBB, fsFCBF, fsFOCUSSF, fsFAST, fsCFSSF, fsSetCover, fsQBB, fsSFFS, fsSBFS, fsMICIBM, fsCHC, fsRanking
} t_FSAlgorithm;

typedef enum {
    frCUB, frFisher
} t_FRAlgorithm;

// Instance+feature selection algorithms
typedef enum {
     NoISFS, CHCGA, IMOEA, ISplusFS, FSplusIS
} t_ISFSAlgorithm;

// Partition methods for scaling
typedef enum {
    pRANDOM, pBOARD, pEQUAL, pRPROTOS, pOneInput, pGrandTour, p2DGrandTour, pkMeans, pkClustering, pDecoupled, pBagging
} t_Partition;

// Ensembling/Boosting feature selection
typedef enum {
    NoBoost, AdaBoost, RealAdaBoost, DiscreteAdaBoost, Wagging, Bagging, RandomSubsampling, GentleBoost, MadaBoost, ReweightBoost, MultiBoost, FloatBoost, BalanceCascade, EasyEnsemble,
    AsymmetricAdaBoost, AdaCost, AdaC1, AdaC2, AdaC3, CSB0, CSB1, CSB2, Arcing, BoostingProj, RandomProj,  AdaBoostMH, FeatureBoost, FreundNN, RotationForest, LearningFromBites, FilterBoost
} t_EnsembleMethod;

// Performance measures
typedef enum {
    pfAccuracy, pfGMean, pfKappa, pfauROC, pfauPRC
} t_PerformanceMeasure;

// Methods for threshold evaluation
typedef enum {
    Thresholding,
    Graph, MFDR, VOOR, MFE, CM // Only available when no instance selection is performed
} t_ThresholdEvaluation;

#endif // _TYPES_H

