/***************************************************************************
 *   Copyright (C) 2009 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <pvm3.h>
#include <sys/file.h>
#include <sys/ipc.h>
#include <sys/types.h>
#include <semaphore.h>
#include <errno.h>
#include "type.h"
#include "macros.h"
#include "common.h"

// Global variables for C4.5 algorithm
unsigned short MaxClass, MaxDiscrVal;
int MaxAtt;
ItemNo MaxItem;
Description *Item;
DiscrValue *MaxAttVal;
char *SpecialStatus;
String *ClassName, *AttName, * *AttValName, FileName;
short VERBOSITY = 0, TRIALS;
Boolean GAINRATIO = true, SUBSET = false;
ItemNo MINOBJS = 2, WINDOW, INCREMENT;
float CF = 0.25;
Tree *Pruned;
Boolean AllKnown = true;

// Outputs of a SVM model already trained

int main(int argc, char *argv[])
{
    int length, window_length, offset;
    char *sequence, model_file[1000], file_name[100];
    t_Recognizer model;
    scalar threshold, value;
    t_Parameters parm;
    struct spectrum_kernel_info spectrum;

    // Test enough parameters.
    if (argc >= 3) {
        // File for output messages
        if ((parm.out_file = fopen("/dev/null", "w")) == NULL)
            OnErrorExit(1, "main: Cannot open output file\n");
        sprintf(file_name, "%s/%s", argv[3], argv[1]);
        LoadGlobalParameters(&parm, file_name);
    } else {
        fprintf(stdout, "brute-force-slave parameters out-file cwd\n");
        exit(1);
    }

    if (parm.r_method != SVM) {
        printf("Only valid for SVM models\n");
        exit(1);
    }

    // Get master TID
    int parent = pvm_parent();

    // Receive parameters
    pvm_recv(parent, SENT_BY_MASTER);
    pvm_upkint(&offset, 1, 1);
    pvm_upkint(&length, 1, 1);
    pvm_upkscalar(&threshold, 1, 1);
    pvm_upkstr(model_file);

    // Receive data
    MALLOC(sequence, char, parm.left_window+length+parm.right_window+1, "main: sequence");
    pvm_upkstr(sequence);

    // Load model
    sprintf(file_name, "%s/%s", argv[3], model_file);
    if ((model.lsvm = svm_load_model(file_name)) == NULL)
        OnErrorExit(1, "Error loading SVM model");

    // File name for output file
    sprintf(file_name, "%s/%s", argv[3], argv[2]);

    window_length = parm.left_window + 3 + parm.right_window;

    // Write classifier outputs
    for (int start = 0; start + window_length < length; start++) {
        spectrum.n_k_mers = -1;
        spectrum.list = spectrum.count = NULL;
        spectrum.k_xx = -1.0;
            
        if ((value = ClassifierOutput((const t_Parameters *) &parm, &model, sequence+start, window_length, 0, NULL, &spectrum)) > threshold) {
            // Positive instance
            // Write to file

            if ((parm.out_file = fopen(file_name, "a")) == NULL)
                OnErrorExit(1, "main: Cannot open output file");

            if (lockf(fileno(parm.out_file), F_LOCK, 0))
                OnErrorExit(1, "Could not lock file");

            char positive_seq[parm.left_window+window_length+parm.right_window+1];
            strncpy(positive_seq, sequence+start, window_length+1);
            positive_seq[window_length+1] = '\0';
            fprintf(parm.out_file, "%lf\t%d\t", value, offset+start);
            for (int i = 0; i < parm.left_window; i++)
                fprintf(parm.out_file, "%c", positive_seq[i]);
            fprintf(parm.out_file, " %c%c%c ", positive_seq[parm.left_window], positive_seq[parm.left_window+1], positive_seq[parm.left_window+2]);
            for (int i = parm.left_window+3; i < window_length; i++)
                fprintf(parm.out_file, "%c", positive_seq[i]);
            fprintf(parm.out_file, "\n");

            if (lockf(fileno(parm.out_file), F_ULOCK, 0))
                OnErrorExit(1, "Could not lock file");

            fclose(parm.out_file);
        }
    }

    FREE(sequence);

    // Send finish to master process
    pvm_initsend(SENT_BY_SLAVE);
    int tid = pvm_mytid();
    pvm_pkint(&tid, 1, 1);
    pvm_send(parent, SENT_BY_SLAVE);
}
