/***************************************************************************
 *   Copyright (C) 2012 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <pvm3.h>
#include <sys/file.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/types.h>
#include <semaphore.h>
#include "type.h"
#include "macros.h"
#include "common.h"

// Outputs of a SVM model already trained
int main(int argc, char *argv[])
{
    int length, *tids;
    char *sequence, file_name[500];
    t_Parameters parm;

    // Test enough parameters.
    if (argc >= 6) {
        // File for output messages.
        if ((parm.out_file = fopen(argv[3], "w")) == NULL)
            OnErrorExit(1, "main: Cannot open output file\n");

        LoadGlobalParameters(&parm, argv[2]);
    } else {
        fprintf(stdout, "brute-force sequences-file parameters-file output-file model-file number-of-slaves\n");
        exit(1);
    }

    if (parm.r_method != SVM) {
        printf("Only valid for SVM models\n");
        exit(1);
    }

    // Named memory for exclusive access to output file
    // lockf() is another alternative
    // char sem_name[1000];
    // sprintf(sem_name, "%s.semaphore", argv[3]);
    // sem_t *mutex = sem_open("", O_CREAT, 0700, sem_name);

    // Read data
    char *home = getenv("HOME");
    strcpy(file_name, argv[1]);
    if (access(file_name, R_OK))
        sprintf(file_name, "%s/%s/%s", home, parm.datad, argv[1]);
    ReadRawFastaFile(file_name, &sequence, &length, parm.max_length);

    // Spawn slaves
    char *args[4];
    MALLOC(args[0], char, strlen(argv[2]) + 1, "main");
    MALLOC(args[1], char, strlen(argv[3]) + 1, "main");
    strcpy(args[0], argv[2]);       // Parameters file name
    strcpy(args[1], argv[3]);       // Output file
    args[2] = getcwd(NULL, 0);      // Current directory for slaves
    args[3] = NULL;

    // Number of slaves to run
    int n_slaves = atoi(argv[5]);

    MALLOC(tids, int, n_slaves, "main: tids");
    // Avoid running processes in master node
    int numt = pvm_spawn("brute-force-slave", args, PvmTaskHost | PvmHostCompl, "skynet.uco.es", n_slaves, tids);
    // int numt = pvm_spawn("brute-force-slave", args, PvmTaskHost | PvmTaskDebug| PvmHostCompl, "skynet.uco.es", n_slaves, tids);

    if (numt != n_slaves)
        OnErrorExit(1, "Processes not spawned correctly");

    // Start guardian and send info
    // int guardian = InitGuardian(numt, tids, "brute-force-slave", argv[3]);

    fclose(parm.out_file);

    // Send data to slaves
    // Offset for fist slave
    int offset = parm.left_window;
    int slave_length = (length-parm.left_window-parm.right_window)/n_slaves;
    for (int i = 0; i < n_slaves; i++) {
        char slave_sequence[slave_length+parm.right_window+parm.left_window+1];

        pvm_initsend(PvmDataDefault);
        pvm_pkint(&offset, 1, 1);
        pvm_pkint(&slave_length, 1, 1);
        pvm_pkscalar(&parm.brute_force_threshold, 1, 1);
        pvm_pkstr(argv[4]);

        strncpy(slave_sequence, sequence+offset-parm.left_window, parm.left_window+slave_length+parm.right_window);
        slave_sequence[parm.left_window+slave_length+parm.right_window] = '\0';
        pvm_pkstr(slave_sequence);
        pvm_send(tids[i], SENT_BY_MASTER);

        offset += slave_length;
    }

    // File for output messages.
    if ((parm.out_file = fopen(argv[3], "a")) == NULL)
        OnErrorExit(1, "main: Cannot open output file\n");

    // Wait for slaves to finish
    for (int i = 0; i < n_slaves; i++) {
        int id;

        pvm_recv(-1, SENT_BY_SLAVE);
        pvm_upkint(&id, 1, 1);

        printf("Finished slave %d\n", id);
    }

    fprintf(parm.out_file, "All slaves finished correctly\n");
    fclose(parm.out_file);

    // Kill guardian
    // pvm_kill(guardian);

    // Remove named semaphore
    // sem_unlink(sem_name);

    return 0;
}
