/***************************************************************************
 *   Copyright (C) 2014 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "type.h"
#include "macros.h"

// Functions for coding sequences as string of bits
// A --> 00
// T --> 01
// G --> 10
// C --> 11
// Other  characters are not allowed and raise an exception when compressing

// Macros for accessing pairs and quartets of bits
#define GETBITPAIR(byte, pair) (((byte) >> (6-(pair)*2)) & 0x03)

// Compress sequence and overwrites old sequence

void CompressSequence(char **sequence)
{
    char *new_sequence;
    int new_length, length;

    length = strlen(*sequence);

    // If the sequence contains only ATGC it is coded in 2 bits, otherwise in left unchanged
    bool only_four_letters = true;
    for (int i = 0; i < strlen(*sequence); i++)
        if ((*sequence)[i] != 'A' && (*sequence)[i] != 'T' && (*sequence)[i] != 'G' && (*sequence)[i] != 'C') {
            only_four_letters = false;
            break;
        }

    // Include space for boolean value that stores whether the sequence is changed
    if (only_four_letters)
        new_length = length%4 ? length/4+2 : length/4+1;
    else
        new_length = length+1;

    CALLOC(new_sequence, char, new_length+1, "CompressSequence: new_sequence");

    // First element is 'T' (compressed) or 'F' (unchanged)
    if (only_four_letters) {
        new_sequence[0] = 'T';
        for (int i = 0; i < length; i++) {
            switch((*sequence)[i]) {
                case 'A':
                    SetBitPair(&new_sequence[i/4+1], i%4, 0);
                    break;

                case 'T':
                    SetBitPair(&new_sequence[i/4+1], i%4, 1);
                    break;

                case 'G':
                    SetBitPair(&new_sequence[i/4+1], i%4, 2);
                    break;

                case 'C':
                    SetBitPair(&new_sequence[i/4+1], i%4, 3);
                    break;
            }
        }
    } else {
        new_sequence[0] = 'F';
        for (int i = 0; i < length; i++)
            new_sequence[i+1] = (*sequence)[i];
    }

    FREE(*sequence);
    *sequence = new_sequence;
}

void UncompressSequence(char *compressed, char *noncompressed, int length)
{
    char letters[] = {'A', 'T', 'G', 'C'};

    // Check coding
    if (compressed[0] == 'T')
        for (int i = 0; i < length; i++)
            noncompressed[i] = letters[GETBITPAIR(compressed[i/4+1], i%4)];
   else
       for (int i = 0; i < length; i++)
            noncompressed[i] = compressed[i+1];

   noncompressed[length] = '\0';
}

void SetBitPair(char *byte, int pair, int value)
    {
        unsigned char clear_mask[4] = {0x3F, 0xCF, 0xF3, 0xFC}, set_mask[4][4] = {{0x00, 0x40, 0x80, 0xC0}, {0x00, 0x10, 0x20, 0x30}, {0x00, 0x04, 0x08, 0x0C}, {0x00, 0x01, 0x02, 0x03}};

        // Clear pair
        *byte &= clear_mask[pair];

        // Set pair
        *byte |= set_mask[pair][value];
    }

int CompressedSequenceLength(char *sequence, int uncompressed_length)
{
    if (sequence[0] == 'T') {
        if (uncompressed_length%4)
            return uncompressed_length/4+2;
        else
            return uncompressed_length/4+1;
    } else
        return uncompressed_length+1;
}
