/***************************************************************************
 *   Copyright (C) 2009 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "type.h"
#include "macros.h"

// Convert sequences to scalar data
void ConvertToScalarData(char **pseqs, int n_pseqs, char **nseqs, int n_nseqs, int length, scalar ***data, int **labels, int *instances, int *features, int kernel)
{
    extern int dna_index[];

    // Allocate data
    *instances = n_pseqs + n_nseqs;

    // If string kernel, the length remains the same
    if (kernel == WD || kernel == WDS || kernel == LI)
        *features = length;
    else
        *features = length * 4;

    MALLOC(*data, scalar *, *instances, "ConvertToScalarData: *data");
    for (int i = 0; i < *instances; i++)
        MALLOC((*data) [i], scalar, *features, "ConvertToScalarData: (*data)[i]");
    MALLOC(*labels, int, *instances, "ConvertToScalarData: *labels");

    // Positive sequences: Class 0
    int count = 0;
    for (int i = 0; i < n_pseqs; i++) {
        // Set inputs
        if (kernel == WD || kernel == WDS || kernel == LI) {
            for (int j = 0; j < length; j++)
                (*data) [count][j] = dna_index [(int) pseqs[i][j]];
        } else {
            // Codify A -> 1000, T -> 0100, G - > 0010, C -> 0001, disregard the rest
            for (int j = 0; j < length; j++)
                switch (pseqs[i][j]) {
                    case 'A':
                        (*data) [count][j*4] = 1.0;
                        (*data) [count][j*4+1] = (*data) [count][j*4+2] = (*data) [count][j*4+3] = 0.0;
                        break;
                    case 'T':
                        (*data) [count][j*4+1] = 1.0;
                        (*data) [count][j*4] = (*data) [count][j*4+2] = (*data) [count][j*4+3] = 0.0;
                        break;
                    case 'G':
                        (*data) [count][j*4+2] = 1.0;
                        (*data) [count][j*4] = (*data) [count][j*4+1] = (*data) [count][j*4+3] = 0.0;
                        break;
                    case 'C':
                        (*data) [count][j*4+3] = 1.0;
                        (*data) [count][j*4] = (*data) [count][j*4+1] = (*data) [count][j*4+2] = 0.0;
                        break;
                    default:
                        (*data) [count][j*4] = (*data) [count][j*4+1] = (*data) [count][j*4+2] = (*data) [count][j*4+3] = 0.0;
                }
        }

        (*labels) [count] = 0;
        count++;
    }

    // Negative sequences: Class 1
    for (int i = 0; i < n_nseqs; i++) {
        // Set inputs
        if (kernel == WD || kernel == WDS || kernel == LI) {
            for (int j = 0; j < length; j++)
                (*data) [count][j] = dna_index [(int) nseqs[i][j]];
        } else {
            // Codify A -> 1000, T -> 0100, G - > 0010, C -> 0001, disregard the rest
            for (int j = 0; j < length; j++)
                switch (nseqs[i][j]) {
                    case 'A':
                        (*data) [count][j*4] = 1.0;
                        (*data) [count][j*4+1] = (*data) [count][j*4+2] = (*data) [count][j*4+3] = 0.0;
                        break;
                    case 'T':
                        (*data) [count][j*4+1] = 1.0;
                        (*data) [count][j*4] = (*data) [count][j*4+2] = (*data) [count][j*4+3] = 0.0;
                        break;
                    case 'G':
                        (*data) [count][j*4+2] = 1.0;
                        (*data) [count][j*4] = (*data) [count][j*4+1] = (*data) [count][j*4+3] = 0.0;
                        break;
                    case 'C':
                        (*data) [count][j*4+3] = 1.0;
                        (*data) [count][j*4] = (*data) [count][j*4+1] = (*data) [count][j*4+2] = 0.0;
                        break;
                    default:
                        (*data) [count][j*4] = (*data) [count][j*4+1] = (*data) [count][j*4+2] = (*data) [count][j*4+3] = 0.0;
                }
        }
        (*labels) [count] = 1;
        count++;
    }
}

// Convert sequences to scalar data
void ConvertToScalarSequence(char *sequence, int length, scalar *instance)
{
    // Codify A -> 1000, T -> 0100, G - > 0010, C -> 0001, disregard the rest
    for (int j = 0; j < length; j++)
        switch (sequence[j]) {
            case 'A':
                instance[j*4] = 1.0;
                instance[j*4+1] = instance[j*4+2] = instance[j*4+3] = 0.0;
                break;
            case 'T':
                instance[j*4+1] = 1.0;
                instance[j*4] = instance[j*4+2] = instance[j*4+3] = 0.0;
                break;
            case 'G':
                instance[j*4+2] = 1.0;
                instance[j*4] = instance[j*4+1] = instance[j*4+3] = 0.0;
                break;
            case 'C':
                instance[j*4+3] = 1.0;
                instance[j*4] = instance[j*4+1] = instance[j*4+2] = 0.0;
                break;
            default:
                instance[j*4] = instance[j*4+1] = instance[j*4+2] = instance[j*4+3] = 0.0;
        }
}
