/***************************************************************************
 *   Copyright (C) 2009 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <wait.h>
#include "type.h"
#include "macros.h"
#include "common.h"

// Global variables for C4.5 algorithm
unsigned short MaxClass, MaxDiscrVal;
int MaxAtt;
ItemNo MaxItem;
Description *Item;
DiscrValue *MaxAttVal;
char *SpecialStatus;
String *ClassName, *AttName, * *AttValName, FileName;
short VERBOSITY = 0, TRIALS;
Boolean GAINRATIO = true, SUBSET = false;
ItemNo MINOBJS = 2, WINDOW, INCREMENT;
float CF = 0.25;
Tree *Pruned;
Boolean AllKnown = true;

int main(int argc, char *argv[])
{
    extern char letters[];

    FILE *data_file;
    time_t init_time, end_time;
    int n_positives, n_negatives, length;
    char **positives, **negatives, file_name[1000], command[1000];
    t_Parameters parm;

    // Test enough parameters
    if (argc >= 5) {
        // File for output messages.
        if ((parm.out_file = fopen(argv[6], "w")) == NULL)
            OnErrorExit(1, "main: Cannot open output file\n");

        LoadGlobalParameters(&parm, argv[3]);
    } else {
        fprintf(stdout, "site-cv-parameters positive-sequences-file negative-sequences-file parameters template-parameters folds output\n");
        exit(1);
    }

    // Use predefined seed or get a new one
    if (parm.seed != 0)
        // Fixed seed to make the evolution reproducible.
        fprintf(parm.out_file, "Random seed: %d (fixed)\n", parm.seed);
    else {
        parm.seed = -time(NULL);
        fprintf(parm.out_file, "Random seed: %d\n", parm.seed);
    }

    srandom(parm.seed);

    // For measuring expended time
    init_time = time(NULL);

    // Read training data
    char *home = getenv("HOME");
    sprintf(file_name, "%s", argv[1]);
    if (access(file_name, R_OK))
        sprintf(file_name, "%s/%s/%s", home, parm.datad, argv[1]);
    ReadFastaFile(file_name, &positives, &n_positives, &length, 1.0, false, 0, false, NULL, 0, false, parm.offset, parm.max_length, parm.sampling_ratio, parm.allow_only_5_letters,
                  parm.do_not_check_sequences, letters, &parm.length, NULL, false);
    fprintf(parm.out_file, "Positive sequences: %d\n", n_positives);

    sprintf(file_name, "%s", argv[2]);
    if (access(file_name, R_OK))
        sprintf(file_name, "%s/%s/%s", home, parm.datad, argv[2]);
    ReadFastaFile(file_name, &negatives, &n_negatives, &length, 1.0, false, 0, false, NULL, 0, false, parm.offset, parm.max_length, parm.sampling_ratio, parm.allow_only_5_letters,
                  parm.do_not_check_sequences, letters, &parm.length, NULL, false);
    fprintf(parm.out_file, "Negative sequences: %d\n", n_negatives);

    // Make folds
    int order_positives[n_positives], order_negatives[n_negatives];

    GetRandomisedVector(order_positives, n_positives);
    GetRandomisedVector(order_negatives, n_negatives);

    // Main loop cross-validation
    // Read template file
    // Template file is in the form
    // Label: value 1, value 2, value 3,..., value N
    FILE *template_file;
    char line[10000], *copy;
    if ((template_file = fopen(argv[4], "r")) == NULL)
        OnErrorExit(1, "Cannot open template file");

    // Count parameters to validate
    if (fgets(line, 10000, template_file) == NULL)
        OnErrorExit(1, "Error reading template file");
    int n_param_to_cv = 0;
    while (!feof(template_file) && line[0] != '\n') {
        if (fgets(line, 10000, template_file) == NULL && !feof(template_file))
            OnErrorExit(1, "Error reading template file");
        n_param_to_cv++;
    }

    // Read values
    char *param_to_cv[n_param_to_cv][100];
    int n_param_values[n_param_to_cv], next_param[n_param_to_cv], best_param[n_param_to_cv];
    MALLOC(copy, char, 10000, "main: copy ");

    rewind(template_file);
    if (fgets(line, 10000, template_file) == NULL)
        OnErrorExit(1, "Error reading template file");
    int c = 0, all_combinations = 1;
    while (!feof(template_file) && line[0] != '\n') {
        // Get values
        RemoveLF(line);
        strcpy(copy, line);
        char *token = strtok(copy, ":");
        n_param_values[c] = 0;
        MALLOC(param_to_cv[c][0], char, strlen(token) + 1, "main: param_to_cv[c][0]")
        strcpy(param_to_cv[c][0], token);
        while ((token = strtok(NULL, ":, ")) != NULL) {
            n_param_values[c]++;
            MALLOC(param_to_cv[c][n_param_values[c]], char, strlen(token) +1, "main: param_to_cv[c][n_param_values[c]]")
            strcpy(param_to_cv[c][n_param_values[c]], token);
        }

        all_combinations *= n_param_values[c];

        if (fgets(line, 10000, template_file) == NULL && !feof(template_file))
            OnErrorExit(1, "Error reading template file");
        c++;
    }

    fclose(template_file);

    // Get base random file name for temporal files
    char *template = tempnam(".", NULL);
    // Remove "./" from the beginning of the template
    template += 2;
    // Iterate for all the combinations
    int n_folds = atoi(argv[5]);

    for (int i = 0; i < n_param_to_cv; i++)
        next_param[i] = 1;

    scalar best = -1.0, auc;
    for (int comb = 0; comb < all_combinations; comb++) {
        // Create parameters file
        sprintf(file_name, "%s.p", template);
        CopyFile(file_name, argv[3]);

        // Add remaining parameters
        if ((template_file = fopen(file_name, "a")) == NULL)
            OnErrorExit(1, "Cannot open template file for appending");
        fprintf(template_file, "[Validation parameters]\nWrite AUC data: T\n");
        for (int i = 0; i < n_param_to_cv; i++)
            fprintf(template_file, "%s: %s\n", param_to_cv[i][0], param_to_cv[i][next_param[i]]);
        fclose(template_file);

        // Iterate for folds
        int pos_fold_size = n_positives / n_folds;
        int neg_fold_size = n_negatives / n_folds;
        auc = 0.0;
        for (int fold = 0; fold < n_folds; fold++) {
            // Create training and test files
            // Positive instances
            int i = 0;
            int j = fold * pos_fold_size;

            // Test file
            sprintf(file_name, "%s.test.pos", template);
            if ((data_file = fopen(file_name, "w")) == NULL)
                OnErrorExit(1, "Cannot open data file");

            for (; i < pos_fold_size; i++, j = (j+1) % n_positives)
                fprintf(data_file, "> %d\n%s\n", j, positives[j]);
            fclose(data_file);

            // Training file
            sprintf(file_name, "%s.train.pos", template);
            if ((data_file = fopen(file_name, "w")) == NULL)
                OnErrorExit(1, "Cannot open data file");
            for (; i < n_positives; i++, j = (j+1) % n_positives)
                fprintf(data_file, "> %d\n%s\n", j, positives[j]);
            fclose(data_file);

            // Negative instances
            i = 0;
            j = fold * neg_fold_size;

            // Testing file
            sprintf(file_name, "%s.test.neg", template);
            if ((data_file = fopen(file_name, "w")) == NULL)
                OnErrorExit(1, "Cannot open data file");

            for (; i < neg_fold_size; i++, j = (j+1) % n_negatives)
                fprintf(data_file, "> %d\n%s\n", j, negatives[j]);
            fclose(data_file);

            // Training file
            sprintf(file_name, "%s.train.neg", template);
            if ((data_file = fopen(file_name, "w")) == NULL)
                OnErrorExit(1, "Cannot open data file");

            for (; i < n_negatives; i++, j = (j+1) % n_negatives)
                fprintf(data_file, "> %d\n%s\n", j, negatives[j]);
            fclose(data_file);

            // Run algorithm
            sprintf(command, "site-recognizer %s.train.pos %s.train.neg %s.test.pos %s.test.neg %s.p %s.out", template, template, template, template, template, template);
            printf("Running: %s\n", command);
            int status = WEXITSTATUS(system(command));

            // If something went wrong exit
            if (status)
                OnErrorExit(1, "Something bad with siterec");

            // Evaluate fold using AUC
            sprintf(file_name, "%s.out-auc", template);
            // Count points in AUC file
            FILE *auc_file;
            int points = -1;
            char line[100];
            if ((auc_file = fopen(file_name, "rt")) == NULL)
                OnErrorExit(1, "Cannot open AUC file");
            do {
                if (fgets(line, 100, auc_file) == NULL && !feof(auc_file))
                    OnErrorExit(1, "Error reading AUC file");
                points++;
            } while (!feof(auc_file));
            fclose(auc_file);
            auc += TrapezoidIntegration(file_name, points);
        }

        // Evaluate and test if best
        auc /= n_folds;

        // Write current evaluation to file
        for (int i = 0; i < n_param_to_cv; i++)
            fprintf(parm.out_file, "%s => %s\t", param_to_cv[i][0], param_to_cv[i][next_param[i]]);
        fprintf(parm.out_file, "AUC = %lf\n", auc);
        fflush(parm.out_file);

        if (auc > best) {
            best = auc;
            for (int i = 0; i < n_param_to_cv; i++)
                best_param[i] = next_param[i];
        }

        printf("DONE, combination %3d/%3d\n", comb+1, all_combinations);

        // Get next set of parameters
        if (!NextSetOfParameters(n_param_to_cv, n_param_values, next_param) && comb != all_combinations - 1)
            OnErrorExit(1, "Something wrong with next set of parameters");
    }

    // Write best parameters file
    // Create parameters file
    sprintf(file_name, "%s.parm.best", argv[6]);
    CopyFile(file_name, argv[3]);

    // Add remaining parameters
    if ((template_file = fopen(file_name, "a")) == NULL)
        OnErrorExit(1, "Cannot open template file for appending");
    fprintf(template_file, "[Validation parameters]\nWrite AUC data: T\n");
    for (int i = 0; i < n_param_to_cv; i++)
        fprintf(template_file, "%s: %s\n", param_to_cv[i][0], param_to_cv[i][best_param[i]]);
    fclose(template_file);

    // Used time
    end_time = time(NULL) - init_time;
    fprintf(parm.out_file, "TRAINING time: %02ld:%02ld:%02ld\n", end_time / 3600, (end_time % 3600) / 60, end_time % 60);
    fprintf(stdout, "TRAINING time: %02ld:%02ld:%02ld\n", end_time / 3600, (end_time % 3600) / 60, end_time % 60);

    // Remove temporarily files
    sprintf(command, "rm %s.*", template);
    int status = WEXITSTATUS(system(command));
    if (status)
        OnErrorExit(1, "Cannot remove temporal files");

    return 0;
}

// Remove '\n''s

void RemoveLF(char *str)
{
    int l = strlen(str) + 1;
    for (int i = 0; i < l; i++)
        if (str[i] == '\n')
            str[i] = '\0';
}
