/***************************************************************************
 *   Copyright (C) 2011 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

// Program for checking running PVM task
// If any of the tasks finishes unexpectedly, shut down the PVM system

#include <unistd.h>
#include <pvm3.h>
#include <stdio.h>
#include <unistd.h>
#include "macros.h"
#include "type.h"

int main(int argc, char *argv[])
{
    int *tids, n_tids;
    char program[1000], gl_out_file[1000], cwd[1000], name[1000];

    // Receive number of tasks
    // Get master TID
    int parent = pvm_parent();
    pvm_recv(parent, SENT_BY_MASTER);

    pvm_upkint(&n_tids, 1, 1);
    ALLOCA(tids, int, n_tids, "main; tids");
    pvm_upkint(tids, n_tids, 1);
    pvm_upkstr(program);
    pvm_upkstr(gl_out_file);
    pvm_upkstr(cwd);

    while (1) {
        // Check no problem with processes
        for (int p = 0; p < n_tids; p++)
            if (pvm_pstat(tids[p]) == PvmNoTask) {
                FILE *out;
                sprintf(name, "%s/%s", cwd, gl_out_file);
                if ((out = fopen(name, "a")) != NULL) {
                    fprintf(out, "TID %d unexpectedly died running program %s\n", tids[p], program);
                    fclose(out);
                }
                pvm_kill(parent);

                printf("TID %d unexpectedly died running program %s\n", tids[p], program);

                // Go away
                exit(1);
            }

        // Sleep a minute
        sleep(60);
    }

    return 0;
}
