/***************************************************************************
 *   Copyright (C) 2009 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <malloc.h>
#include "type.h"
#include "macros.h"

// Undersampling methods for class imbalance problems

// Undersample randomly the majority class until both classes have the same number of instances
void UndersampleData(char *** data, int *n, int minority_n, bool free_data)
{
    int CompareInteger(const void *, const void *);

    int *list;

    // Get a random sub-sample of majority class
    MALLOC(list, int, *n, "UndersampleData: list");
    for (int i = 0; i < *n; i++)
        list[i] = i;
    GetRandomisedVector(list, *n);
    qsort(list, minority_n, sizeof(int), CompareInteger);

    // Exchange with the first ones
    for (int i = 0; i < minority_n;  i++) {
        // Majority class instance
        char *aux = (*data) [i];
        (*data) [i] = (*data) [list[i]];
        (*data) [list[i]] = aux;
    }

    FREE(list);

    // Reallocate memory and free data if apply
    if (free_data) {
        for (int i = minority_n; i < *n; i++)
            FREE((*data) [i]);
        REALLOC(*data, char *, minority_n, "UndersampleData: *data");
    }

    *n = minority_n;
}
