/***************************************************************************
 *   Copyright (C) 2009 by Nicolas Garcia-Pedrajas   *
 *   npedrajas@uco.es   *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "type.h"
#include "common.h"
#include "macros.h"

void LoadGlobalParameters(t_Parameters *parm, char *f_name)
{
    FILE *f;
    char *recognition[] = {"SVM", "kNN", "C45", "STOP", "EasyEnsembleEnsemble", "BalanceCascadeEnsemble", "PWM", "NeuralNet", "LearningFromBites"},
         *imbalance[] = {"NONE", "UNDERSAMPLING", "SMOTE"},
         *fsalgorithm[] = {"NONE", "RFE", "GA" },
         *noise_filer[] = {"NONE", "GEditing"};
    int n_recognition = 9, n_imbalance = 3, n_stackers = 2, n_fsalgorithm = 3, n_base_learners = 3, n_noise_filer = 2;

    // Open file
    if ((f = fopen(f_name, "r")) == NULL) {
        printf("Parameters file: %s\n", f_name);
        OnErrorExit(ERR_FILE, "Cannot open parameters file");
    }

    fprintf(parm->out_file, "Parameters (* = default) \n----------\n");

    // Data
    LoadString(f, "Data directory", "/home/frodo/Data/Bioinformatics", parm->datad, parm->out_file);
    parm->imbalance_method = LoadEnumerate(f, "Imbalance method", imbalance, n_imbalance, (int) iNone, parm->out_file);
    parm->eval_training = LoadBoolean(f, "Evaluate training error", false, parm->out_file);
    parm->roc_training = LoadBoolean(f, "Obtain ROC for training data", false, parm->out_file);

    // Sequence
    parm->offset = LoadInteger(f, "Offset to start", 0, parm->out_file);
    parm->offset_to_site = LoadInteger(f, "Offset to site", 500, parm->out_file);
    parm->length = LoadInteger(f, "Sequence length", 0, parm->out_file);          // If 0 read to finish of sequence
    parm->allow_different_lengths = LoadBoolean(f, "Allow different lengths", false, parm->out_file);
    parm->max_length = LoadInteger(f, "Maximum length", 10000, parm->out_file);    // Only valid for allow different lengths
    parm->use_protein_sequence = LoadBoolean(f, "Use protein sequence", false, parm->out_file);
    LoadString(f, "Mask file", "nil" , parm->mask_file, parm->out_file);
    parm->sampling_ratio = LoadReal(f, "Sampling ratio", 1.0, parm->out_file);     // For large files do not use all instances
    parm->allow_only_5_letters = LoadBoolean(f, "Allow only 5 letters", false, parm->out_file); // Other than ATGC is substituted by N
    parm->do_not_check_sequences = LoadBoolean(f, "Do not check sequences", false, parm->out_file);
    parm->use_salzberg_method = LoadBoolean(f, "Use Salzberg method", false, parm->out_file);
    parm->max_positive_instances = LoadInteger(f, "Maximum number of positive instances", 0, parm->out_file);

    // General
    parm->verbose = LoadInteger(f, "Verbose", 0, parm->out_file);
    parm->seed = LoadInteger(f, "Seed", 0, parm->out_file);
    parm->r_method = LoadEnumerate(f, "Recognition method", recognition, n_recognition, (int) STOP, parm->out_file);
    parm->sensitivity = LoadReal(f, "Desired sensitivity", 0.8, parm->out_file);
    parm->cv_parameters = LoadBoolean(f, "Cross-validate parameters", true, parm->out_file);
    parm->write_auc_data = LoadBoolean(f, "Write AUC data", false, parm->out_file);

    // Noise filtering
    parm->noise_filter = LoadEnumerate(f, "Noise filter", noise_filer, n_noise_filer, (int) nfNone, parm->out_file);

    // Feature selection
    parm->fsalgorithm = LoadEnumerate(f, "FS algorithm", fsalgorithm, n_fsalgorithm, (int) fsNONE, parm->out_file);
    parm->rfe_subset_size = LoadInteger(f, "RFE subset size", 10, parm->out_file);
    parm->fs_percent_to_keep = LoadReal(f, "FS percent to keep", 0.0, parm->out_file);
    parm->cv_folds = LoadInteger(f, "CV folds", 10, parm->out_file);
    parm->fs_min_percent = LoadReal(f, "FS minimum percent", 0.5, parm->out_file);
    parm->exit_after_fs = LoadBoolean(f, "Exit after FS", false, parm->out_file);

    // STOP codon method
    parm->reverse_stop_codon = LoadBoolean(f, "Reverse STOP codon", false, parm->out_file); // For detecting stop codons

    // Stacking
    parm->stacking = LoadBoolean(f, "Stacking", false, parm->out_file);
    parm->stacking_methods = LoadInteger(f, "Stacking method", 71, parm->out_file);
    // 511(0x1FF):  ALL
    // 71(0x47):   STOP + C4.5 + SVM + kNN
    // 199(0xC7):  STOP + C4.5 + SVM + kNN + PWM
    // 127(0x7F):  STOP + C4.5 + SVM + SVM(WD) + Easy + Balance cascade + kNN
    // 119(0x77):  STOP + C4.5 + SVM + Easy + Balance cascade + kNN
    // 247 (0xF7): STOP + C4.5 + SVM + Easy + Balance cascade + kNN + PWM
    // 79(0x4F):   STOP + C4.5 + SVM + SVM(WD) + kNN
    // 7(0x7):     STOP + C4.5 + SVM
    // 204 (0xCC): SVM + SVM(WD) + kNN + PWM
    parm->scale = LoadBoolean(f, "Scale stacking inputs", true, parm->out_file);
    parm->stacker = LoadEnumerate(f, "Stacker", recognition, n_stackers, (int) SVM, parm->out_file);
    if (parm->stacking)
        parm->r_method = NoOne;

    // C4.5
    TRIALS = LoadInteger(f, "Trials", 1, parm->out_file);
    parm->prune_tree = LoadBoolean(f, "Prune tree", true, parm->out_file);
    parm->soften_thresholds = LoadBoolean(f, "Soften thresholds", true, parm->out_file);

    // SVM
    // enum { 0: LINEAR, 1: POLY, 2: RBF, 3: SIGMOID, 4: PRECOMPUTED, 5: WD, 6: WDS, 7:LI, 8:SPECTRUM, 9:MISMATCH, 10:SMITHWATERMAN, 11:WILDCARD }; /* kernel type */
    parm->kernel = LoadInteger(f, "Kernel", 0, parm->out_file);
    parm->c = LoadReal(f, "C", 1.0, parm->out_file);
    switch (parm->kernel) {
        case LINEAR:
        case POLY:
        case RBF:
        case PRECOMPUTED:
        case SIGMOID:
            parm->degree = LoadReal(f, "Degree", 1.0, parm->out_file);
            parm->gamma = LoadReal(f, "Gamma", 1.0, parm->out_file);
            break;

        case WD:
            parm->degree = LoadInteger(f, "Degree (WD)", 12, parm->out_file);         // d parameter
            break;

        case WDS:
            parm->degree = LoadInteger(f, "Degree (WDS)", 12, parm->out_file);         // d parameter
            parm->gamma = LoadReal(f, "Sigma (WDS)", 0.5, parm->out_file);          // sigma parameter
            break;

        case LI:
            parm->degree = LoadInteger(f, "Degree (LI)", 3, parm->out_file);
            parm->gamma = LoadReal(f, "l (LI)", 3.0, parm->out_file);

        case SPECTRUM:
            parm->degree = LoadInteger(f, "k (SPECTRUM)", 6, parm->out_file);
            // Hybrid kernel. 0: None, 1: Linear product, 2: Linear sum, 3: Gaussian product; 4: Gaussian sum
            parm->offset = LoadInteger(f, "Hybrid kernel (SPECTRUM)", 0, parm->out_file);
            parm->coef0 = LoadReal(f, "Gamma (SPECTRUM)", 1.0, parm->out_file);
            if (parm->offset < 0 && parm->offset > 4)
                OnErrorExit(ERR_PARAM, "Hybrid kernel not valid");
            // printf("Do not forget to assign to gamma the number of letters of the alphabet\n");
            parm->gamma = LoadReal(f, "Number of tokens (SPECTRUM)", 18, parm->out_file);
            break;

        case MISMATCH:
            parm->degree = LoadInteger(f, "k (MISMATCH)", 4, parm->out_file);
            parm->offset = LoadInteger(f, "m (MISMATCH)", 1, parm->out_file);
            // printf("Do not forget to assign to gamma the number of letters of the alphabet\n");
            parm->gamma = LoadReal(f, "Number of tokens (MISMATCH)", 18, parm->out_file);
            break;

        case SMITHWATERMAN:
            parm->gamma = LoadReal(f, "Mu (SMITH-WATERMAN)", 0.33, parm->out_file);
            parm->coef0 = LoadReal(f, "Delta (SMITH-WATERMAN)", 1.33, parm->out_file);
            bool use_minimum = LoadBoolean(f, "Use minimum (SMITH-WATERMAN)", true, parm->out_file);
            parm->degree = use_minimum ? 1 : 0;
            break;

        case WILDCARD:
            parm->degree = LoadInteger(f, "k (WILDCARD)", 4, parm->out_file);
            parm->offset = LoadInteger(f, "m (WILDCARD)", 1, parm->out_file);
            parm->coef0 = LoadReal(f, "lambda (WILDCARD)", 1.0, parm->out_file);
            // printf("Do not forget to assign to gamma the number of letters of the alphabet\n");
            parm->gamma = LoadReal(f, "Number of tokens (WILDCARD)", 18, parm->out_file);
            break;
    }

    // k-NN
    parm->k = LoadInteger(f, "k", 3, parm->out_file);
    parm->min_k = LoadInteger(f, "Minimum k", 3, parm->out_file);
    parm->max_k = LoadInteger(f, "Maximum k", 25, parm->out_file);
    parm->knn_weight_by_distance = LoadBoolean(f, "KNN weight by distance", true, parm->out_file);

    // Ensemble
    parm->base_learner = LoadEnumerate(f, "Base learner", recognition, n_base_learners, (int) C45, parm->out_file);
    parm->ensemble_t = LoadInteger(f, "Ensemble t", 10, parm->out_file);
    parm->easy_s_i = LoadInteger(f, "Ensemble s_i", 10, parm->out_file);

    // Learning from bites
    parm->n_bites = LoadInteger(f, "Number of bites", 1000, parm->out_file);
    parm->bite_size = LoadInteger(f, "Bite size", 100, parm->out_file);

    // Smote
    parm->smote_k = LoadInteger(f, "SMOTE k", 5, parm->out_file);
    parm->smote_n = LoadReal(f, "SMOTE N", 100, parm->out_file);

    // Neural network
    parm->nn_hidden_nodes = LoadInteger(f, "Network hidden nodes", 100, parm->out_file);
    parm->n_epochs = LoadInteger(f, "N epochs", 100000, parm->out_file);
    parm->epoch_size = LoadInteger(f, "Epoch size", 16, parm->out_file);
    parm->eta = LoadInteger(f, "Eta", 0.75, parm->out_file);
    parm->alpha = LoadInteger(f, "Alpha", 0.25, parm->out_file);
    parm->htan_a = LoadReal(f, "Htan a", 1.0, parm->out_file);
    parm->htan_b = LoadReal(f, "Htan b", 0.3333, parm->out_file);
    parm->flat_spot = LoadReal(f, "Flat spot", 0.01, parm->out_file);

    // GA for feature selection
    parm->gafs_alpha = LoadReal(f, "GA alpha", 0.5, parm->out_file);
    parm->gafs_mutation = LoadReal(f, "GA mutation", 0.1, parm->out_file);
    parm->gafs_lsmutation = LoadReal(f, "GA LS mutation", 0.1, parm->out_file);
    parm->gafs_bit_mutation = LoadReal(f, "GA bit mutation", 0.1, parm->out_file);
    parm->gafs_set_probability = LoadReal(f, "GA 1 probability", 0.75, parm->out_file);
    parm->gafs_popsize = LoadInteger(f, "GA population size", 100, parm->out_file);
    parm->gafs_generations = LoadInteger(f, "GA generations", 1000, parm->out_file);

    // General editing noise filtering
    parm->ge_k = LoadInteger(f, "General editing k", 3, parm->out_file);
    parm->ge_kprime = LoadInteger(f, "General editing k\'", 2, parm->out_file);

    // Stepwise procedures
    parm->plus_l = LoadInteger(f, "LRS L", 3, parm->out_file);
    parm->minus_r = LoadInteger(f, "LRS R", 1, parm->out_file);
    parm->stepwise_max_attempts = LoadInteger(f, "Stepwise maximum attempts", 250, parm->out_file);

    fclose(f);

    // Incompatibilities and special cases
    if (parm->r_method == EasyEnsembleEnsemble || parm->r_method == BalanceCascadeEnsemble)
        // Undersampling included in the method
        parm->imbalance_method = iNone;

    if (parm->base_learner != C45 && parm->base_learner != SVM)
        OnErrorExit(ERR_PARAM, "Ensembles only with C4.5 or SVMs");

    if (parm->fsalgorithm != fsNONE && parm->r_method == STOP)
        OnErrorExit(ERR_PARAM, "Stop codon method does not accept feature selection");

    if (parm->fs_percent_to_keep > 1.0) {
        parm->fs_percent_to_keep = 1.0;
        printf("Percent to keep above 100%%, corrected\n");
    }

    // Protein sequence only for WD kernels
    if (parm->use_protein_sequence && (parm->r_method != SVM || parm->kernel != WD))
        OnErrorExit(ERR_PARAM, "Protein sequence only for string kernels");

    // If kernel = WD, no validation can be applied
    if (parm->kernel == WD || parm->kernel == WDS || parm->kernel == LI)
        parm->cv_parameters = false;

    // If k=0 for SMOTE choose k according to N
    if (parm->smote_k == 0)
        parm->smote_k = (int) parm->smote_n / 100;

    if (parm->stacking) {
        if (!((parm->stacking_methods & stSTOP) || (parm->stacking_methods & stC45) || (parm->stacking_methods & stSVM) || (parm->stacking_methods & stSVMSTR) ||
            (parm->stacking_methods & stEasy) || (parm->stacking_methods & stCascade) || (parm->stacking_methods & stKNN)))
            // Check at least one classifier
            OnErrorExit(ERR_PARAM, "No stacked classifiers");
    }

    if (parm->use_salzberg_method) {
        printf("Adjusting parameters for Salzberg method\n");
        parm->stacking = false;
        parm->r_method = SVM;
        parm->use_protein_sequence = false;
        if (parm->imbalance_method == iSmote)
            parm->imbalance_method = iNone;
        parm->fsalgorithm = fsNONE;
        parm->cv_parameters = false;
    }

    if (parm->r_method == STOP && parm->verbose > 1) {
        if (!parm->reverse_stop_codon)
            printf("WARNING: For stop codon method the sequence must start with ATG\n");
        else
            printf("WARNING: For reverse stop codon method the sequence must end with TAG, TAA or TGA\n");
    }

    if (parm->allow_different_lengths) {
        if ((parm->r_method != SVM || (parm->kernel != WD && parm->kernel != WDS && parm->kernel != SPECTRUM && parm->kernel != SMITHWATERMAN && parm->kernel != MISMATCH && parm->kernel != WILDCARD))
            || parm->stacking || parm->fsalgorithm != fsNONE)
        OnErrorExit(ERR_PARAM, "Different lengths only allowed for string kernels (WD for SVMs)");
    }

    if (parm->ge_kprime > parm->ge_k || parm->ge_kprime < (parm->ge_k+1)/2)
        OnErrorExit(ERR_PARAM, "Bad value for General Editing k\'");

    if (parm->minus_r <= 0 || parm->plus_l <= 0)
	OnErrorExit(ERR_PARAM, "LRS selection method bad parameters");
}
