/***************************************************************************
 *   Copyright (C) 2009 by Nicolas Garcia-Pedrajas   *
 *   npedrajas@uco.es   *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdbool.h>
#include <stdio.h>
#include <dirent.h>
#include <string.h>
#include <math.h>
#include "macros.h"

void AddModelsToFileName(char *file_name, bool *selected_models, int n_models)
{
    bool first;

    // Nine first models
    for (int i = 0; i < n_models && i < 9; i++)
        if (selected_models[i])
            sprintf(file_name, "%s_%d", file_name, i+1);

    for (int i = 9; i < n_models; i += 10) {
        first = true;

        for (int j = 0; j < 10 && i+j < n_models; j++)
            if (selected_models[i+j]) {
                if (first) {
                    first = false;
                    sprintf(file_name, "%s_%d.%d", file_name, (i+1)/10, j);
                } else
                    sprintf(file_name, "%s%d", file_name, j);
            }
    }

    // Check maximum size exceeded. Then truncate.
    if (strlen(file_name) >= MAXNAMLEN)
        file_name[MAXNAMLEN] = '\0';
}

int ConvertToNumberWithoutMask(int number, int *usable_models, int n_usable)
{
    int converted = 0;

    for (int i = 0; i < n_usable; i++) {
        if (number % 2)
            converted += (int) pow(2.0, usable_models[i]);
        number /= 2;
    }

    return converted;
}

void RandomizeSequences(char **seqs, int n)
{
    int l, r;

    for (int i = 0; i < n; i++) {
        // Exchange two random positions
        l = RANDOM(0, n);
        r = RANDOM(0, n);

        char *aux = seqs[l];
        seqs[l] = seqs[r];
        seqs[r] = aux;
    }
}
