/***************************************************************************
 *   Copyright (C) 2009 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <pvm3.h>
#include <sys/file.h>
#include <sys/ipc.h>
#include <sys/types.h>
#include <semaphore.h>
#include <errno.h>
#include "type.h"
#include "macros.h"
#include "common.h"

// Slave for the outputs of a SVM model already trained

// Global variables for C4.5 algorithm
unsigned short MaxClass, MaxDiscrVal;
int MaxAtt;
ItemNo MaxItem;
Description *Item;
DiscrValue *MaxAttVal;
char *SpecialStatus;
String *ClassName, *AttName, * *AttValName, FileName;
short VERBOSITY = 0, TRIALS;
Boolean GAINRATIO = true, SUBSET = false;
ItemNo MINOBJS = 2, WINDOW, INCREMENT;
float CF = 0.25;
Tree *Pruned;
Boolean AllKnown = true;

int main(int argc, char *argv[])
{
    int seqs_sent, instance_index;
    char *sequence[SEQUENCES_BLOCK], file_name[1000];
    t_Recognizer model;
    scalar value[SEQUENCES_BLOCK];
    t_Parameters parm;
    struct spectrum_kernel_info spectrum;

    // Test enough parameters.
    if (argc >= 3) {
        // Output messages to null
        if ((parm.out_file = fopen("/dev/null", "w")) == NULL)
            OnErrorExit(1, "main: Cannot open output file\n");
        // Load parameters from file
        // Check where it is
        strcpy(file_name, argv[1]);
        if (access(file_name, R_OK))
            sprintf(file_name, "%s/%s", argv[3], argv[1]);

        LoadGlobalParameters(&parm, file_name);
    } else {
        fprintf(stdout, "model-output-slave parameters-file model-file cwd\n");
        exit(1);
    }

    for (int i = 0; i < SEQUENCES_BLOCK; i++)
        MALLOC(sequence[i], char, MAX_SEQUENCE_LENGTH, "main: sequence[i]");

    // Get master TID
    int parent = pvm_parent();

    // Load model
    switch (parm.r_method) {
        case SVM:
        case kNN:
        case STOP:
        case PWM:
        case C45:
            // Check where it is
            strcpy(file_name, argv[2]);
            if (access(file_name, R_OK))
                sprintf(file_name, "%s/%s", argv[3], argv[2]);

            LoadModel(&parm, &model, file_name);
            printf("Model read from file: %s\n", file_name);
            break;

        default:
            printf("Not valid for this model\n");
            exit(1);
    }

    // Write classifier outputs
    while (1) {
        // Receive sequence
        pvm_recv(parent, SENT_BY_MASTER);
        pvm_upkint(&seqs_sent, 1, 1);
        pvm_upkint(&instance_index, 1, 1);
        for (int i = 0; i < seqs_sent; i++)
            pvm_upkstr(sequence[i]);

        // Classify
        for (int i = 0; i < seqs_sent; i++) {
            spectrum.n_k_mers = -1;
            spectrum.list = spectrum.count = NULL;
            spectrum.k_xx = -1.0;
            value[i] = ClassifierOutput((const t_Parameters *) &parm, &model, sequence[i], strlen(sequence[i]), 0, NULL, &spectrum);
        }

        pvm_initsend(PvmDataDefault);
        pvm_pkint(&seqs_sent, 1, 1);
        pvm_pkint(&instance_index, 1, 1);
        for (int i = 0; i < seqs_sent; i++)
            pvm_pkscalar(&value[i], 1, 1);
        pvm_send(parent, SENT_BY_SLAVE);
    }

    return 0;
}
