/***************************************************************************
 *   Copyright (C) 2009 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <pvm3.h>
#include <time.h>
#include "type.h"
#include "macros.h"
#include "common.h"

// #define VERBOSE_OUTPUT(str, integer)    printf("%s: %d\n", str, integer)
#define VERBOSE_OUTPUT(str, integer)    ;

// Global variables for C4.5 algorithm
unsigned short MaxClass, MaxDiscrVal;
int MaxAtt;
ItemNo MaxItem;
Description *Item;
DiscrValue *MaxAttVal;
char *SpecialStatus;
String *ClassName, *AttName, * *AttValName, FileName;
short VERBOSITY = 0, TRIALS;
Boolean GAINRATIO = true, SUBSET = false;
ItemNo MINOBJS = 2, WINDOW, INCREMENT;
float CF = 0.25;
Tree *Pruned;
Boolean AllKnown = true;

int main(int argc, char *argv[])
{
    extern char letters[];

    int n_positives, n_negatives = 0, length, *tids = NULL, guardian = -1, count = 0;
    char **positives, **negatives = NULL, file_name[1000];
    t_Recognizer R;
    FILE *save;
    t_Parameters parm;
    scalar *outputs = NULL;

    // Test enough parameters
    if (argc >= 6) {
        // File for output messages.
        if ((parm.out_file = fopen(argv[4], "w")) == NULL)
            OnErrorExit(1, "main: Cannot open output file\n");

        LoadGlobalParameters(&parm, argv[3]);
    } else {
        fprintf(stdout, "model-output seqs-positives seqs-negatives|NULL parameters output model-file [number-of-slaves]\n");
        exit(1);
    }

    if (parm.r_method != SVM && parm.r_method != kNN && parm.r_method != STOP && parm.r_method != PWM && parm.r_method != C45) {
        printf("Only valid for SVM models\n");
        exit(1);
    }

    // Read training data
    char *home = getenv("HOME");
    sprintf(file_name, "%s", argv[1]);
    if (access(file_name, R_OK))
        sprintf(file_name, "%s/%s/%s", home, parm.datad, argv[1]);
    // No file, no positive sequences
    if (access(file_name, R_OK))
        n_positives = 0;
    else
        ReadFastaFile(file_name, &positives, &n_positives, &length, 1.0, false, 0, false, NULL, 0, false, parm.offset, parm.max_length, parm.sampling_ratio, parm.allow_only_5_letters,
                      parm.do_not_check_sequences, letters, &parm.length, NULL, false);
    fprintf(parm.out_file, "Positive sequences: %d\n", n_positives);

    if (strncmp(argv[2], "NULL", 4)) {
        sprintf(file_name, "%s", argv[2]);
        if (access(file_name, R_OK))
            sprintf(file_name, "%s/%s/%s", home, parm.datad, argv[2]);
        // No file, no negative sequences
        if (access(file_name, R_OK))
            n_negatives = 0;
        else
            ReadFastaFile(file_name, &negatives, &n_negatives, &length, 1.0, false, 0, false, NULL, 0, false, parm.offset, parm.max_length, parm.sampling_ratio, parm.allow_only_5_letters,
                        parm.do_not_check_sequences, letters, &parm.length, NULL, false);
    }
    fprintf(parm.out_file, "Negative sequences: %d\n", n_negatives);

    // Mask inputs if apply
    int *mask = NULL;
    if (strncmp(parm.mask_file, "nil", 3)) {
        // Read mask file, testing there is a value for each base
        FILE *maskf;
        char line[100];

        MALLOC(mask, int, length, "main: mask");

        if ((maskf = fopen(parm.mask_file, "r")) == NULL)
            OnErrorExit(1, "Cannot open mask file");

        int base = 0, c = 0;
        if (fgets(line, 100, maskf) == NULL)
            OnErrorExit(1, "Error reading mask file");
        while (!feof(maskf) && base < length) {
            mask[base] = atoi(line);

            // Count masked variables
            if (!mask[base])
                c++;

            base++;
            if (fgets(line, 100, maskf) == NULL && !feof(maskf))
                OnErrorExit(1, "Error reading mask file");
        }
        fclose(maskf);

        if (base != length)
            OnErrorExit(1, "Unexpected finish of mask file");

        if (c == length)
            OnErrorExit(1, "All bases masked out");

        // Mask with N's
        for (int i = 0; i < n_positives; i++)
            for (int j = 0; j < length; j++)
                if (!mask[j])
                    positives[i][j] = 'N';

        for (int i = 0; i < n_negatives; i++)
            for (int j = 0; j < length; j++)
                if (!mask[j])
                    negatives[i][j] = 'N';
    }

    // Allocate space for outputs
    MALLOC(outputs, scalar, n_positives+n_negatives, "main: outputs");

    // Number of slaves in case of a parallel version
    int n_slaves = 0;
    if (argc >= 7)
        n_slaves = atoi(argv[6]);

    if (n_slaves > 0) {
        char *args[4];
        MALLOC(args[0], char, strlen(argv[3]) + 1, "main");
        MALLOC(args[1], char, strlen(argv[5]) + 1, "main");
        strcpy(args[0], argv[3]);       // Parameters file name
        strcpy(args[1], argv[5]);       // Model file name
        args[2] = getcwd(NULL, 0);      // Current directory for slaves
        args[3] = NULL;

        MALLOC(tids, int, n_slaves, "main: tids");
        // Avoid running processes in master node
        int numt = pvm_spawn("model-output-slave", args, PvmTaskHost | PvmHostCompl, "skynet.uco.es", n_slaves, tids);
        // int numt = pvm_spawn("model-output-slave", args, PvmTaskHost | PvmTaskDebug| PvmHostCompl, "skynet.uco.es", n_slaves, tids);

        if (numt != n_slaves)
            OnErrorExit(1, "Processes not spawned correctly");

        // Start guardian and send info
        guardian = InitGuardian(numt, tids, "model-output-slave", argv[4]);
    }

    // Write classifier outputs
    char par_file_name[10000];
    // Remove directories from parameters file name
    int index = strlen(argv[3])-1;
    while (index >= 0 && argv[3][index] != '/')
        index--;
    strcpy(par_file_name, argv[3]+index+1);

    sprintf(file_name, "%s+%s.%s.out", argv[1], argv[2], par_file_name);
    if ((save = fopen(file_name, "w")) == NULL)
        OnErrorExit(1, "Cannot open file for saving outputs");

    scalar TP, TN, FP, FN, value;
    TP = TN = FP = FN = 0.0;

    // For measuring expended time
    time_t init_time = time(NULL);

    // Load model for arguments of the classifier
    LoadModel(&parm, &R, argv[5]);
    printf("Model loaded\n");

    // Sequential or parallel version?
    if (n_slaves <= 0) {
        struct spectrum_kernel_info *spectrum = NULL;
        if (parm.kernel == SPECTRUM || parm.kernel == MISMATCH || parm.kernel == WILDCARD) {
            MALLOC(spectrum, struct spectrum_kernel_info, n_positives+n_negatives, "main: spectrum");
            for (int i = 0; i < n_positives+n_negatives; i++) {
                spectrum[i].n_k_mers = -1;
                spectrum[i].list = NULL;
                spectrum[i].count = NULL;
                spectrum[i].k_xx = -1.0;
            }
        }

        for (int i = 0; i < n_positives; i++) {
            fprintf(save, "%7.4lf%s\n", value = ClassifierOutput((const t_Parameters *) &parm, &R, positives[i], length, 0, NULL, &spectrum[i]), n_negatives ? " 1" : "");
            outputs[count] = value;
            count++;

            if (value >= R.threshold)
                TP++;
            else
                FN++;

            VERBOSE_OUTPUT("Positive instance", i);
        }

        for (int i = 0; i < n_negatives; i++) {
            fprintf(save, "%7.4lf 0\n", value = ClassifierOutput((const t_Parameters *) &parm, &R, negatives[i], length, 0, NULL, &spectrum[i+n_positives]));
            outputs[count] = value;
            count++;

            if (value < R.threshold)
                TN++;
            else
                FP++;

            VERBOSE_OUTPUT("Negative instance", i);
        }
    } else {
        int seqs_sent = 1, sent, received, first_instance;
        for (int i = 0; i < n_positives; i += n_slaves) {
            // Send to slaves
            for (int j = 0; j < n_slaves && i+j < n_positives; j++) {
                pvm_initsend(PvmDataDefault);
                pvm_pkint(&seqs_sent, 1, 1);
                sent = i+j;
                pvm_pkint(&sent, 1, 1); // Index of first pattern
                pvm_pkstr(positives[i+j]);
                // pvm_pkbyte(positives[i+j], length+1, 1);
                pvm_send(tids[j], SENT_BY_MASTER);
            }

            // Receive classification from slaves
            for (int j = 0; j < n_slaves && i+j < n_positives; j++) {
                pvm_recv(-1, SENT_BY_SLAVE);
                pvm_upkint(&seqs_sent, 1, 1);
                pvm_upkint(&first_instance, 1, 1);
                pvm_upkscalar(&value, 1, 1);

                outputs[first_instance] = value;
                count++;
                if (value >= R.threshold)
                    TP++;
                else
                    FN++;

                VERBOSE_OUTPUT("Positive instance", i+j);
            }
        }

        for (int i = 0; i < n_positives; i++)
             fprintf(save, "%7.4lf%s\n", outputs[i], n_negatives ? " 1" : "");

        sent = received = 0;

        // Send to slaves a first round for all of them
        for (int j = 0; j < n_slaves && sent < n_negatives; j++) {
            // Sequences to sent
            seqs_sent = MIN(SEQUENCES_BLOCK, n_negatives-sent);
            pvm_initsend(PvmDataDefault);
            pvm_pkint(&seqs_sent, 1, 1);
            pvm_pkint(&sent, 1, 1);
            for (int k = 0; k < seqs_sent; k++, sent++)
                pvm_pkstr(negatives[sent]);
                // pvm_pkbyte(negatives[sent], length+1, 1);
            pvm_send(tids[j], SENT_BY_MASTER);
        }

        while (received < n_negatives) {
            // Receive classification from any slave
            int bufid = pvm_recv(-1, SENT_BY_SLAVE);

            // If still more instances, send to this slave
            if (sent < n_negatives) {
                // Get TID of sending slave
                int bytes, mstag, tid;
                pvm_bufinfo(bufid, &bytes, &mstag, &tid);

                // Sequences to sent
                seqs_sent = MIN(SEQUENCES_BLOCK, n_negatives-sent);
                pvm_initsend(PvmDataDefault);
                pvm_pkint(&seqs_sent, 1, 1);
                pvm_pkint(&sent, 1, 1); // Index of first pattern
                for (int k = 0; k < seqs_sent; k++, sent++)
                    pvm_pkstr(negatives[sent]);
                    // pvm_pkbyte(negatives[sent], length+1, 1);
                pvm_send(tid, SENT_BY_MASTER);
            }

            pvm_upkint(&seqs_sent, 1, 1);
            pvm_upkint(&first_instance, 1, 1);
            for (int k = 0; k < seqs_sent; k++, received++) {
                pvm_upkscalar(&value, 1, 1);

                outputs[first_instance+k+n_positives] = value;
                count++;
                if (value < R.threshold)
                    TN++;
                else
                    FP++;
                VERBOSE_OUTPUT("Negative  instance", received);
            }
        }

        for (int i = 0; i < n_negatives; i++)
             fprintf(save, "%7.4lf 0\n", outputs[i+n_positives]);

        // Kill slaves
        pvm_kill(guardian);
        for (int i = 0; i < n_slaves; i++)
            pvm_kill(tids[i]);
    }
    fclose(save);

    // Free training data
    for (int i = 0; i < n_positives; i++)
        FREE(positives[i]);
    if (n_positives)
        FREE(positives);
    for (int i = 0; i < n_negatives; i++)
        FREE(negatives[i]);
    if (n_negatives)
        FREE(negatives);

    scalar sn = TP+FN > 0.0 ? TP/(TP+FN) : 0.0;
    scalar sp = TN+FP > 0.0 ? TN/(TN+FP) : 0.0;
    scalar precision = TP+FP > 0 ? TP/(TP+FP) : 0.0;

    fprintf(parm.out_file, "@TP: %d\n@FN: %d\n@TN: %d\n@FP: %d\n", (int) TP, (int) FN, (int) TN, (int) FP);
    fprintf(parm.out_file, "@Sensitivity (TP rate): %lf\n", sn);
    fprintf(parm.out_file, "@Specificity (TN rate): %lf\n", sp);
    fprintf(parm.out_file, "@Bioinformatics specificity: %lf\n", TP+FP > 0.0 ? TP/(TP+FP) : 0.0);
    fprintf(parm.out_file, "@G-measure: %lf\n@F-measure: %lf\n@Precision: %lf\n", sqrt(sn*sp), sn+precision > 0.0 ? (2.0*sn*precision)/(sn+precision) : 0.0, precision);
    fprintf(parm.out_file, "@FP rate: %lf\n", FP+TN > 0.0 ? FP/(FP+TN) : 0.0);
    fprintf(parm.out_file, "@FN rate: %lf\n", FN/n_positives);
    fprintf(parm.out_file, "@auROC: %lf\n", AUROCFromOutputs(outputs, n_positives, n_negatives, false, NULL, false));
    fprintf(parm.out_file, "@auPRC: %lf\n", AUPRCFromOutputs(outputs, n_positives, n_negatives, false, NULL, true));

    // Used time
    time_t end_time = time(NULL) - init_time;
    fprintf(parm.out_file, "@Time: %02ld:%02ld:%02ld\n", end_time/3600, (end_time % 3600)/60, end_time % 60);
    fprintf(parm.out_file, "@Ok\n");

    fclose(parm.out_file);
}
