/***************************************************************************
 *   Copyright (C) 2013 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "type.h"

#define LARGE	1.E100

// Ranks of models in a previously sorted output summary

int main(int argc, char *argv[])
{
    FILE *f;
	char line[1000];

    if (argc < 3) {
        printf("models-ranking combinations-file number-of-models\n");
        exit(1);
    }

    if ((f = fopen(argv[1], "r")) == NULL) {
		perror ("Problem with input file");
		exit(1);
	}

	// Initirsedxalize rankings
	int n_of_models = atoi(argv[2]);

	double rankings[n_of_models];
	int counts[n_of_models];
	for (int i = 0; i < n_of_models; i++) {
		rankings[i] = 0.0;
		counts[i] = 0;
	}

	int n = 0;

        if (fgets(line, 1000, f) == NULL && !feof(f))
            OnErrorExit(1, "Error reading data file");
	while (!feof(f)) {
		n++;

		// Parse line
		char *models = strstr(line, "Models: ");

		// Avoid space
		if (models[0] == ' ')
			models++;

		// Parse models
		char *parsed = strtok(models, " ");
		// Ignore "Models: "
		parsed = strtok(NULL, " ");
		do {
			int model = atoi(parsed);

			if (model < 1 || model > n_of_models) {
				printf("%d: %s\n", n, parsed);
				printf("Model out of range: %d\n", model);
				exit(1);
			}

			rankings[model-1] += n;
			counts[model-1]++;

		} while((parsed = strtok(NULL, " ")));

		// New line
                if (fgets(line, 1000, f) == NULL && !feof(f))
                    OnErrorExit(1, "Error reading data file");
	}

	fclose (f);

	// Average values
	for (int i = 0; i < n_of_models; i++)
		rankings[i] /= counts[i];

	// Print sorted ranks
	printf("Order\tModel\tPresent\tAverage ranking\n");
	for (int i = 0; i < n_of_models; i++) {
		int best = 0;

		for (int j = 1; j < n_of_models; j++)
			if (rankings[best] > rankings[j])
				best = j;

		printf("%2d\t%2d\t%6d\t%10.2lf\n", i+1, best+1, counts[best], rankings[best]);

		// Do not consider it again
		rankings[best] = LARGE;
	}

	return 0;
}
