/***************************************************************************
 *   Copyright (C) 2011 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
// Functions that deal with protein sequences
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include "type.h"
#include "macros.h"

char *code[][2] = {{"TTT", "F"}, {"TTC", "F"}, {"TTA", "L"}, {"TTG", "L"}, {"CTT", "L"}, {"CTC", "L"}, {"CTA", "L"}, {"CTG", "L"},
    {"ATT", "I"}, {"ATC", "I"}, {"ATA", "I"}, {"ATG", "M"}, {"GTT", "V"}, {"GTC", "V"}, {"GTA", "V"}, {"GTG", "V"},
    {"TCT", "S"}, {"TCC", "S"}, {"TCA", "S"}, {"TCG", "S"}, {"CCT", "P"}, {"CCC", "P"}, {"CCA", "P"}, {"CCG", "P"},
    {"ACT", "T"}, {"ACC", "T"}, {"ACA", "T"}, {"ACG", "T"}, {"GCT", "A"}, {"GCC", "A"}, {"GCA", "A"}, {"GCG", "A"},
    {"TAT", "Y"}, {"TAC", "Y"}, {"TAA", "X"}, {"TAG", "X"}, {"CAT", "H"}, {"CAC", "H"}, {"CAA", "Q"}, {"CAG", "Q"},
    {"AAT", "N"}, {"AAC", "N"}, {"AAA", "K"}, {"AAG", "K"}, {"GAT", "D"}, {"GAC", "D"}, {"GAA", "E"}, {"GAG", "E"},
    {"TGT", "C"}, {"TGC", "C"}, {"TGA", "X"}, {"TGG", "W"}, {"CGT", "R"}, {"CGC", "R"}, {"CGA", "R"}, {"CGG", "R"},
    {"AGT", "S"}, {"AGC", "S"}, {"AGA", "R"}, {"AGG", "R"}, {"GGT", "G"}, {"GGC", "G"}, {"GGA", "G"}, {"GGG", "G"}
};

// Convert a sequence from DNA to protein. Must be in-frame

int ConvertToProtein(char **seqs, int n_seqs, int length)
{
    // Check multiple of three
    if (length % 3)
        OnErrorExit(1, "Cannot convert to protein if incomplete codons");

    for (int i = 0; i < n_seqs; i++) {
        // Translate every codon
        int codon = 0;
        for (int j = 0; j < length; j += 3) {
            bool found = false;

            for (int k = 0; k < 64 && !found; k++)
                if (!strncasecmp(seqs[i]+j, code[k][0], 3)) {
                    // Assign amino acid
                    seqs[i][codon] = code[k][1][0];
                    codon++;
                    found = true;
                }

            // If not found assign 'X'
            if (!found) {
                seqs[i][codon] = 'X';
                codon++;
            }
        }

        // Reallocate memory
        REALLOC(seqs[i], char, length/3 + 1, "ConvertToProtein");
        seqs[i][length/3] = '\0';
    }

    // New length
    return length/3;
}
