/***************************************************************************
 *   Copyright (C) 2012 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

// Convert string to dinucleotide frequency
#include "macros.h"
#include "type.h"

void ConvertToDinucleotideFrequency(char ***positives, int n_positives, char ***negatives, int n_negatives, scalar ***new_poss, scalar ***new_negs, int *length, scalar ***frequencies)
{
    char *dinucleotide[] = {"AA", "AT", "AG", "AC", "TA", "TT", "TG", "TC", "GA", "GT", "GG", "GC", "CA", "CT", "CG", "CC"};

    // If no previous frequencies obtain
    if (*frequencies == NULL) {
        MALLOC(*frequencies, scalar *, 16, "ConvertToDinucleotideFrequency: *frequencies");
        for (int i = 0; i < 16; i++)
            MALLOC((*frequencies)[i], scalar, *length-1, "ConvertToDinucleotideFrequency: (*frequencies)[i]");

        // Use pseudocounts
        for (int i = 0; i < *length-1; i++)
            for (int j = 0; j < 16; j++) {
                scalar counts_positive = 1.;
                for (int s = 0; s < n_positives; s++)
                    if ((*positives)[s][i] == dinucleotide[j][0] && (*positives)[s][i+1] == dinucleotide[j][1])
                        counts_positive++;

                scalar counts_negative = 1.;
                for (int s = 0; s < n_negatives; s++)
                    if ((*negatives)[s][i] == dinucleotide[j][0] && (*negatives)[s][i+1] == dinucleotide[j][1])
                        counts_negative++;

                (*frequencies)[j][i] = counts_positive/(counts_positive+counts_negative);
            }
    }

    // Allocate memory for new data
    (*length)--;
    MALLOC(*new_poss, scalar *, n_positives, "ConvertToDinucleotideFrequency: *new_poss");
    for (int i = 0; i < n_positives; i++) {
        MALLOC((*new_poss)[i], scalar, *length, "ConvertToDinucleotideFrequency: (*new_poss)[i]");
        for (int j = 0; j < *length; j++) {
            // Contains different letters from ATGC
            if (((*positives)[i][j] != 'A' && (*positives)[i][j] != 'T' && (*positives)[i][j] != 'G' && (*positives)[i][j] != 'C') ||
                ((*positives)[i][j+1] != 'A' && (*positives)[i][j+1] != 'T' && (*positives)[i][j+1] != 'G' && (*positives)[i][j+1] != 'C')) {
                (*new_poss)[i][j] = 0.0;
            } else {
                int di = -1;
                do {
                    di++;
                } while ((*positives)[i][j] != dinucleotide[di][0] || (*positives)[i][j+1] != dinucleotide[di][1]);

                (*new_poss)[i][j] = (*frequencies)[di][j];
            }
        }

        FREE((*positives)[i]);
    }

    MALLOC(*new_negs, scalar *, n_negatives, "ConvertToDinucleotideFrequency: *new_negs");
    for (int i = 0; i < n_negatives; i++) {
        MALLOC((*new_negs)[i], scalar, *length, "ConvertToDinucleotideFrequency: (*new_negs)[i]");
        for (int j = 0; j < *length; j++) {
            if (((*negatives)[i][j] != 'A' && (*negatives)[i][j] != 'T' && (*negatives)[i][j] != 'G' && (*negatives)[i][j] != 'C') ||
                ((*negatives)[i][j+1] != 'A' && (*negatives)[i][j+1] != 'T' && (*negatives)[i][j+1] != 'G' && (*negatives)[i][j+1] != 'C')) {
                (*new_poss)[i][j] = 0.0;
            } else {
                int di = -1;
                do {
                    di++;
                } while ((*negatives)[i][j] != dinucleotide[di][0] || (*negatives)[i][j+1] != dinucleotide[di][1]);

                (*new_negs)[i][j] = (*frequencies)[di][j];
            }
        }

        FREE((*negatives)[i]);
    }

    // Free previous data
    FREE(*positives);
    FREE(*negatives);
}
