/***************************************************************************
 *   Copyright (C) 2011 by Nicolas Garcia-Pedrajas   *
 *   npedrajas@uco.es   *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "type.h"
#include "macros.h"

void ScaleData(scalar **data, int n, int inputs, scalar lbound, scalar ubound, scalar **a, scalar **b, bool scale)
{
    scalar max_val, min_val;   /* Make the linear transformation x' = ax + b,
                                  where a=(ubound - lbound)/(max_val-min_val)
                                  b= (lbound · max_val- ubound ·min_val)/(max_val-min_val) */

    // Memory for coefficients
    MALLOC(*a, scalar, inputs, "ScaleData: *a");
    MALLOC(*b, scalar, inputs, "ScaleData: *b");

    // If no scaling, just fill vectors
    if (!scale) {
        for (int i = 0; i < inputs; i++) {
            (*a)[i] = 1.0;
            (*b)[i] = 0.0;
        }

        return;
    }

    // Scale
    for (int i = 0; i < inputs; i++) {
        // Get the max and min values of the column
        min_val = max_val = data[0][i];
        for (int j = 0; j < n; j++)
            if (max_val < data[j][i])
                max_val = data[j][i];
            else {
                if (min_val > data[j][i])
                    min_val = data[j][i];
            }

        // Calculate a and b coefficients
        // If constant do nothing.
        if (fabs(max_val - min_val) > ZERO) {
            (*a)[i] = (ubound - lbound) / (max_val - min_val);
            (*b)[i] = (lbound * max_val - ubound * min_val) / (max_val - min_val);
        } else {
            (*a)[i] = 1.0;
            (*b)[i] = 0.0;
        }

        // Scale
        for (int j = 0; j < n; j++)
            data[j][i] = (*a)[i] * data[j][i] + (*b)[i];
    }
}
