/***************************************************************************
 *   Copyright (C) 2016 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "type.h"
#include "macros.h"
#include "common.h"

// Global variables for C4.5 algorithm
unsigned short MaxClass, MaxDiscrVal;
int MaxAtt;
ItemNo MaxItem;
Description *Item;
DiscrValue *MaxAttVal;
char *SpecialStatus;
String *ClassName, *AttName, * *AttValName, FileName;
short VERBOSITY = 0, TRIALS;
Boolean GAINRATIO = true, SUBSET = false;
ItemNo MINOBJS = 2, WINDOW, INCREMENT;
float CF = 0.25;
Tree *Pruned;
Boolean AllKnown = true;

// Write the result of a model for just one sequence
// Call: seq-model-output model-file parameters-file sequence

int main(int argc, char *argv[])
{
    t_Parameters parm;
    t_Recognizer R;
    struct spectrum_kernel_info spectrum;
    bool verbose = false, use_threshold = false;

    if (argc < 5) {
        printf("seq-model-output model-file parameters-file sequence [use_threshold = {y, n*}] [verbose = {y, n*}]\n");
        exit(1);
    }

    if (argc == 6 && tolower(argv[5][0]) == 'y')
        verbose = true;

    if (tolower(argv[4][0]) == 'y')
        use_threshold = true;

    // File for output messages.
    if ((parm.out_file = fopen("/dev/null", "w")) == NULL)
        OnErrorExit(1, "main: Cannot open /dev/null file\n");

    // Load parameters
    LoadGlobalParameters(&parm, argv[2]);

    // Load model
    LoadModel(&parm, &R, argv[1]);

    if (verbose) {
        fprintf(stderr, "Read model file %s\n", argv[1]);
        fprintf(stderr, "Threshold: %lf\n", R.threshold);
    }

    // Cut sequence appropriately
    char sequence[strlen(argv[3])+1];
    strncpy(sequence, argv[3]+parm.offset, parm.length);

    if (verbose)
        printf("Sequence: %s\n", sequence);

    spectrum.n_k_mers = -1;
    spectrum.list = spectrum.count = NULL;
    spectrum.k_xx = -1.0;
    scalar output = ClassifierOutput((const t_Parameters *) &parm, &R, sequence, parm.length, 0, NULL, &spectrum);

    if (!use_threshold)
        R.threshold = 0.0;

    if (verbose) {
        printf("Classifier output: %lf\n", output);
        printf("Output: %lf\n", output - R.threshold);
    } else
        printf("%lf", output - R.threshold);

    return 0;
}
