/***************************************************************************
 *   Copyright (C) 2009 by Nicolas Garcia-Pedrajas                         *
 *   npedrajas@uco.es                                                      *
 *                                                                         *
 *   This program is FREE software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

// Functions that deal with support vector machines
#include <math.h>
#include <string.h>
#include "type.h"
#include "common.h"
#include "svm.h"

scalar SVMOutput(struct svm_model *smodel, char *sequence, int length, scalar *sc_seq, bool use_salzberg_method, struct spectrum_kernel_info *spectrum)
{
    double dec_values;
    struct svm_node svm_x[length + 1];
    int in;

    extern int dna_index[];

    // Create description
    in = 0;
    if (use_salzberg_method) {
        for (int j = 0; j < length; j++) {
            svm_x[in].index = j;
            svm_x[in].value = sc_seq[j];
            in++;
        }
    } else if (smodel->param.kernel_type != WD && smodel->param.kernel_type != WDS && smodel->param.kernel_type != LI && smodel->param.kernel_type != SPECTRUM && smodel->param.kernel_type != MISMATCH
                && smodel->param.kernel_type != SMITHWATERMAN && smodel->param.kernel_type != WILDCARD) {
        for (int j = 0; j < length; j++)
            if (dna_index[(int) sequence[j]] != -1) {
                svm_x[in].index = j * 4 + dna_index[(int) sequence[j]];
                svm_x[in].value = 1.0;
                in++;
            }
    } else {
        for (int j = 0; j < length && j < strlen(sequence); j++) {
            svm_x[in].index = j;
            svm_x[in].value = dna_index[(int) sequence[j]];
            in++;
        }
    }

    svm_x[in].index = -1;
    svm_x[0].spectrum = spectrum;

    // Get output
    // scalar class = svm_predict(smodel, svm_x);
    svm_predict_values(smodel, svm_x, &dec_values);

    // Return class
    return (scalar) dec_values;
}
