/***************************************************************************
*   Copyright (C) 20089 by Nicolas Garcia-Pedrajas                         *
*   npedrajas@uco.es                                                      *
*                                                                         *
*   This program is FREE software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <alloca.h>
#include "type.h"
#include "defns.i"
#include "types.i"
#include "extern.i"
#include "macros.h"
#include "common.h"

/*
  Return vector w of: D(X) = w.x + b for each pair of classes.

  w[k(k-1)/2][inputs]
*/

scalar ** GetSVMW(struct svm_model *model, int inputs)
{
    int k, *start;
    scalar **w, **SVs, *coeffs;

    void Product(scalar *, scalar **, scalar *, int, int);

    // Allocate memory
    k = model->nr_class * (model->nr_class - 1) / 2;
    MALLOC(w, scalar *, k, "GetSVMW: w");
    MALLOC(SVs, scalar *, model->l, "GetSVMW: SVs");
    for (int i = 0; i < model->l; i++)
        MALLOC(SVs[i], scalar, inputs, "GetSVMW: SVs[i]");
    MALLOC(coeffs, scalar, model->l, "GetSVMW: coeffs");
    MALLOC(start, int, model->nr_class, "GetSVMW: start");
    for (int i = 0; i < k; i++)
        MALLOC(w[i], scalar, inputs, "GetSVMW: w[i]");

    start[0] = 0;
    for (int i = 1; i < model->nr_class; i++)
        start[i] = start[i - 1] + model->nSV[i - 1];

    // Get w[] for each pair of classes
    int n = 0;
    for (int i = 0; i < model->nr_class; i++)
        for (int j = i + 1; j < model->nr_class; j++) {
            // Get "transposed" matrix of support vectors of classes
            // and vector of coefficients for i and j
            int nSVs = 0;
            for (int l = start[i]; l < start[i] + model->nSV[i]; l++) {
                // Copy support vector
                int m = 0;
                for (int o = 0; o < inputs; o++)
                    if (model->SV[l][m].index == o) {
                        SVs[nSVs][o] = model->SV[l][m].value;
                        m++;
                    } else
                        SVs[nSVs][o] = 0.0;

                coeffs[nSVs] = model->sv_coef[j-1][l];
                nSVs++;
            }

            for (int l = start[j]; l < start[j] + model->nSV[j]; l++) {
                // Copy support vector
                int m = 0;
                for (int o = 0; o < inputs; o++)
                    if (model->SV[l][m].index == o) {
                        SVs[nSVs][o] = model->SV[l][m].value;
                        m++;
                    } else
                        SVs[nSVs][o] = 0.0;

                coeffs[nSVs] = model->sv_coef[i][l];
                nSVs++;
            }

            // Matrix product to obtain w[]
            Product(coeffs, SVs, w[n], nSVs, inputs);
            n++;
        }
    for (int i = 0; i < model->l; i++)
        FREE(SVs[i]);
    FREE(SVs);
    FREE(coeffs);
    FREE(start);

    return w;
}

// Obtains: c = AB, where gl_c and A are vectors, a B is rxc

void
Product(scalar *A, scalar **B, scalar *gl_c, int r, int c)
{
    for (int j = 0; j < c; j++) {
        gl_c[j] = A[0] * B[0][j];
        for (int h = 1; h < r; h++)
            gl_c[j] += A[h] * B[h][j];
    }
}
